/* 
 *  Unix SMB/Netbios implementation.
 *  Version 3.0
 *  error mapping functions
 *  Copyright (C) Andrew Tridgell 2001
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "includes.h"

/* NT status -> dos error map */
static struct {
	uint8 dos_class;
	uint32 dos_code;
	NTSTATUS ntstatus;
} ntstatus_to_dos_map[] = {
	{ERRDOS,	997,	NT_STATUS(0x00000103)},
	{ERRDOS,	111,	STATUS_MORE_ENTRIES},
	{ERRDOS,	1300,	NT_STATUS(0x00000106)},
	{ERRDOS,	1301,	NT_STATUS(0x00000107)},
	{ERRDOS,	1022,	NT_STATUS(0x0000010c)},
	{ERRDOS,	1302,	NT_STATUS(0x0000010d)},
	{ERRDOS,	8201,	NT_STATUS(0x00000121)},
	{ERRDOS,	31,	NT_STATUS_UNSUCCESSFUL},
	{ERRSRV,	ERRsmbcmd,	NT_STATUS_NOT_IMPLEMENTED},
	{ERRDOS,	ERRbadpipe,	NT_STATUS_INVALID_INFO_CLASS},
	{ERRDOS,	24,	NT_STATUS_INFO_LENGTH_MISMATCH},
	{ERRDOS,	998,	NT_STATUS_ACCESS_VIOLATION},
	{ERRDOS,	999,	NT_STATUS_IN_PAGE_ERROR},
	{ERRDOS,	1454,	NT_STATUS_PAGEFILE_QUOTA},
	{ERRDOS,	ERRbadfid,	NT_STATUS_INVALID_HANDLE},
	{ERRDOS,	1001,	NT_STATUS_BAD_INITIAL_STACK},
	{ERRDOS,	193,	NT_STATUS_BAD_INITIAL_PC},
	{ERRDOS,	ERRbadpipe,	NT_STATUS_INVALID_CID},
	{ERRDOS,	ERRbadpipe,	NT_STATUS_INVALID_PARAMETER},
	{ERRDOS,	ERRbadfile,	NT_STATUS_NO_SUCH_DEVICE},
	{ERRDOS,	ERRbadfile,	NT_STATUS_NO_SUCH_FILE},
	{ERRSRV,	ERRsmbcmd,	NT_STATUS_INVALID_DEVICE_REQUEST},
	{ERRDOS,	38,	NT_STATUS_END_OF_FILE},
	{ERRDOS,	34,	NT_STATUS_WRONG_VOLUME},
	{ERRDOS,	21,	NT_STATUS_NO_MEDIA_IN_DEVICE},
	{ERRDOS,	1785,	NT_STATUS_UNRECOGNIZED_MEDIA},
	{ERRDOS,	27,	NT_STATUS_NONEXISTENT_SECTOR},
	{ERRDOS,	111,	NT_STATUS_MORE_PROCESSING_REQUIRED},
	{ERRDOS,	ERRnomem,	NT_STATUS_NO_MEMORY},
	{ERRDOS,	487,	NT_STATUS_CONFLICTING_ADDRESSES},
	{ERRDOS,	487,	NT_STATUS_NOT_MAPPED_VIEW},
	{ERRDOS,	ERRbadpipe,	NT_STATUS(0xc000001a)},
	{ERRDOS,	ERRbadpipe,	NT_STATUS_UNABLE_TO_DELETE_SECTION},
	{ERRSRV,	ERRsmbcmd,	NT_STATUS_INVALID_SYSTEM_SERVICE},
	{ERRDOS,	29,	NT_STATUS_ILLEGAL_INSTRUCTION},
	{ERRDOS,	ERRbadaccess,	NT_STATUS_INVALID_LOCK_SEQUENCE},
	{ERRDOS,	ERRbadaccess,	NT_STATUS_INVALID_VIEW_SIZE},
	{ERRDOS,	193,	NT_STATUS_INVALID_FILE_FOR_SECTION},
	{ERRDOS,	ERRbadaccess,	NT_STATUS_ALREADY_COMMITTED},
	{ERRDOS,	ERRbadaccess,	NT_STATUS_ACCESS_DENIED},
	{ERRDOS,	ERRinsufficientbuffer,	NT_STATUS_BUFFER_TOO_SMALL},
	{ERRDOS,	ERRbadfid,	NT_STATUS_OBJECT_TYPE_MISMATCH},
	{ERRDOS,	37,	NT_STATUS_NONCONTINUABLE_EXCEPTION},
	{ERRDOS,	38,	NT_STATUS_INVALID_DISPOSITION},
	{ERRDOS,	158,	NT_STATUS_NOT_LOCKED},
	{ERRDOS,	43,	NT_STATUS_PARITY_ERROR},
	{ERRDOS,	487,	NT_STATUS_UNABLE_TO_DECOMMIT_VM},
	{ERRDOS,	487,	NT_STATUS_NOT_COMMITTED},
	{ERRDOS,	ERRbadpipe,	NT_STATUS_INVALID_PARAMETER_MIX},
	{ERRDOS,	26,	NT_STATUS_DISK_CORRUPT_ERROR},
	{ERRSRV,	206,	NT_STATUS_OBJECT_NAME_INVALID},
	{ERRDOS,	ERRbadfile,	NT_STATUS_OBJECT_NAME_NOT_FOUND},
	{ERRDOS,	ERRfilexists,	NT_STATUS_OBJECT_NAME_COLLISION},
	{ERRDOS,	ERRbadfid,	NT_STATUS_PORT_DISCONNECTED},
	{ERRDOS,	161,	NT_STATUS_OBJECT_PATH_INVALID},
	{ERRDOS,	ERRbadpath,	NT_STATUS_OBJECT_PATH_NOT_FOUND},
	{ERRDOS,	161,	NT_STATUS_OBJECT_PATH_SYNTAX_BAD},
	{ERRDOS,	1117,	NT_STATUS_DATA_OVERRUN},
	{ERRDOS,	1117,	NT_STATUS_DATA_LATE_ERROR},
	{ERRDOS,	ERRbaddata,	NT_STATUS_DATA_ERROR},
	{ERRDOS,	ERRbaddata,	NT_STATUS_CRC_ERROR},
	{ERRDOS,	ERRnomem,	NT_STATUS_SECTION_TOO_BIG},
	{ERRDOS,	ERRbadaccess,	NT_STATUS_PORT_CONNECTION_REFUSED},
	{ERRDOS,	ERRbadfid,	NT_STATUS_INVALID_PORT_HANDLE},
	{ERRDOS,	ERRbadshare,	NT_STATUS_SHARING_VIOLATION},
	{ERRDOS,	1816,	NT_STATUS_QUOTA_EXCEEDED},
	{ERRDOS,	ERRbadpipe,	NT_STATUS_INVALID_PAGE_PROTECTION},
	{ERRDOS,	288,	NT_STATUS_MUTANT_NOT_OWNED},
	{ERRDOS,	298,	NT_STATUS_SEMAPHORE_LIMIT_EXCEEDED},
	{ERRDOS,	ERRbadpipe,	NT_STATUS_PORT_ALREADY_SET},
	{ERRDOS,	ERRbadpipe,	NT_STATUS_SECTION_NOT_IMAGE},
	{ERRDOS,	156,	NT_STATUS_SUSPEND_COUNT_EXCEEDED},
	{ERRDOS,	ERRbadaccess,	NT_STATUS_THREAD_IS_TERMINATING},
	{ERRDOS,	ERRbadpipe,	NT_STATUS_BAD_WORKING_SET_LIMIT},
	{ERRDOS,	ERRbadpipe,	NT_STATUS_INCOMPATIBLE_FILE_MAP},
	{ERRDOS,	ERRbadpipe,	NT_STATUS_SECTION_PROTECTION},
	{ERRDOS,	282,	NT_STATUS_EAS_NOT_SUPPORTED},
	{ERRDOS,	275,	NT_STATUS_EA_TOO_LARGE},
	{ERRDOS,	276,	NT_STATUS_NONEXISTENT_EA_ENTRY},
	{ERRDOS,	276,	NT_STATUS_NO_EAS_ON_FILE},
	{ERRDOS,	276,	NT_STATUS_EA_CORRUPT_ERROR},
	{ERRDOS,	ERRlock,	NT_STATUS_FILE_LOCK_CONFLICT},
	{ERRDOS,	ERRlock,	NT_STATUS_LOCK_NOT_GRANTED},
	{ERRDOS,	ERRbadaccess,	NT_STATUS_DELETE_PENDING},
	{ERRDOS,	ERRunsup,	NT_STATUS_CTL_FILE_NOT_SUPPORTED},
	{ERRDOS,	1305,	NT_STATUS_UNKNOWN_REVISION},
	{ERRDOS,	1306,	NT_STATUS_REVISION_MISMATCH},
	{ERRDOS,	1307,	NT_STATUS_INVALID_OWNER},
	{ERRDOS,	1308,	NT_STATUS_INVALID_PRIMARY_GROUP},
	{ERRDOS,	1309,	NT_STATUS_NO_IMPERSONATION_TOKEN},
	{ERRDOS,	1310,	NT_STATUS_CANT_DISABLE_MANDATORY},
	{ERRDOS,	1311,	NT_STATUS_NO_LOGON_SERVERS},
	{ERRDOS,	1312,	NT_STATUS_NO_SUCH_LOGON_SESSION},
	{ERRDOS,	1313,	NT_STATUS_NO_SUCH_PRIVILEGE},
	{ERRDOS,	1314,	NT_STATUS_PRIVILEGE_NOT_HELD},
	{ERRDOS,	1315,	NT_STATUS_INVALID_ACCOUNT_NAME},
	{ERRDOS,	1316,	NT_STATUS_USER_EXISTS},
	{ERRDOS,	1317,	NT_STATUS_NO_SUCH_USER},
	{ERRDOS,	1318,	NT_STATUS_GROUP_EXISTS},
	{ERRDOS,	1319,	NT_STATUS_NO_SUCH_GROUP},
	{ERRDOS,	1320,	NT_STATUS_MEMBER_IN_GROUP},
	{ERRDOS,	1321,	NT_STATUS_MEMBER_NOT_IN_GROUP},
	{ERRDOS,	1322,	NT_STATUS_LAST_ADMIN},
	{ERRSRV,	ERRbadpw,	NT_STATUS_WRONG_PASSWORD},
	{ERRDOS,	1324,	NT_STATUS_ILL_FORMED_PASSWORD},
	{ERRDOS,	1325,	NT_STATUS_PASSWORD_RESTRICTION},
	{ERRDOS,	ERRlogonfailure, NT_STATUS_LOGON_FAILURE},
	{ERRDOS,	1327,	NT_STATUS_ACCOUNT_RESTRICTION},
	{ERRSRV,	2241,	NT_STATUS_INVALID_LOGON_HOURS},
	{ERRSRV,	2240,	NT_STATUS_INVALID_WORKSTATION},
	{ERRSRV,	2242,	NT_STATUS_PASSWORD_EXPIRED},
	{ERRSRV,	2239,	NT_STATUS_ACCOUNT_DISABLED},
	{ERRDOS,	1332,	NT_STATUS_NONE_MAPPED},
	{ERRDOS,	1333,	NT_STATUS_TOO_MANY_LUIDS_REQUESTED},
	{ERRDOS,	1334,	NT_STATUS_LUIDS_EXHAUSTED},
	{ERRDOS,	1335,	NT_STATUS_INVALID_SUB_AUTHORITY},
	{ERRDOS,	1336,	NT_STATUS_INVALID_ACL},
	{ERRDOS,	1337,	NT_STATUS_INVALID_SID},
	{ERRDOS,	1338,	NT_STATUS_INVALID_SECURITY_DESCR},
	{ERRDOS,	127,	NT_STATUS_PROCEDURE_NOT_FOUND},
	{ERRDOS,	193,	NT_STATUS_INVALID_IMAGE_FORMAT},
	{ERRDOS,	1008,	NT_STATUS_NO_TOKEN},
	{ERRDOS,	1340,	NT_STATUS_BAD_INHERITANCE_ACL},
	{ERRDOS,	158,	NT_STATUS_RANGE_NOT_LOCKED},
	{ERRSRV,	ERRnoroom,	NT_STATUS_DISK_FULL},
	{ERRDOS,	1341,	NT_STATUS_SERVER_DISABLED},
	{ERRDOS,	1342,	NT_STATUS_SERVER_NOT_DISABLED},
	{ERRDOS,	68,	NT_STATUS_TOO_MANY_GUIDS_REQUESTED},
	{ERRDOS,	259,	NT_STATUS_GUIDS_EXHAUSTED},
	{ERRDOS,	1343,	NT_STATUS_INVALID_ID_AUTHORITY},
	{ERRDOS,	259,	NT_STATUS_AGENTS_EXHAUSTED},
	{ERRDOS,	154,	NT_STATUS_INVALID_VOLUME_LABEL},
	{ERRDOS,	14,	NT_STATUS_SECTION_NOT_EXTENDED},
	{ERRDOS,	487,	NT_STATUS_NOT_MAPPED_DATA},
	{ERRDOS,	1812,	NT_STATUS_RESOURCE_DATA_NOT_FOUND},
	{ERRDOS,	1813,	NT_STATUS_RESOURCE_TYPE_NOT_FOUND},
	{ERRDOS,	1814,	NT_STATUS_RESOURCE_NAME_NOT_FOUND},
	{ERRDOS,	140,	NT_STATUS_ARRAY_BOUNDS_EXCEEDED},
	{ERRDOS,	141,	NT_STATUS_FLOAT_DENORMAL_OPERAND},
	{ERRDOS,	142,	NT_STATUS_FLOAT_DIVIDE_BY_ZERO},
	{ERRDOS,	143,	NT_STATUS_FLOAT_INEXACT_RESULT},
	{ERRDOS,	144,	NT_STATUS_FLOAT_INVALID_OPERATION},
	{ERRDOS,	145,	NT_STATUS_FLOAT_OVERFLOW},
	{ERRDOS,	146,	NT_STATUS_FLOAT_STACK_CHECK},
	{ERRDOS,	147,	NT_STATUS_FLOAT_UNDERFLOW},
	{ERRDOS,	148,	NT_STATUS_INTEGER_DIVIDE_BY_ZERO},
	{ERRDOS,	534,	NT_STATUS_INTEGER_OVERFLOW},
	{ERRDOS,	150,	NT_STATUS_PRIVILEGED_INSTRUCTION},
	{ERRDOS,	ERRnomem,	NT_STATUS_TOO_MANY_PAGING_FILES},
	{ERRDOS,	1006,	NT_STATUS_FILE_INVALID},
	{ERRDOS,	1344,	NT_STATUS_ALLOTTED_SPACE_EXCEEDED},
	{ERRDOS,	ERRnomem,	NT_STATUS_INSUFFICIENT_RESOURCES},
	{ERRDOS,	ERRbadpath,	NT_STATUS_DFS_EXIT_PATH_FOUND},
	{ERRDOS,	ERRbaddata,	NT_STATUS_DEVICE_DATA_ERROR},
	{ERRDOS,	1167,	NT_STATUS_DEVICE_NOT_CONNECTED},
	{ERRDOS,	21,	NT_STATUS_DEVICE_POWER_FAILURE},
	{ERRDOS,	487,	NT_STATUS_FREE_VM_NOT_AT_BASE},
	{ERRDOS,	487,	NT_STATUS_MEMORY_NOT_ALLOCATED},
	{ERRDOS,	1453,	NT_STATUS_WORKING_SET_QUOTA},
	{ERRDOS,	19,	NT_STATUS_MEDIA_WRITE_PROTECTED},
	{ERRDOS,	21,	NT_STATUS_DEVICE_NOT_READY},
	{ERRDOS,	1345,	NT_STATUS_INVALID_GROUP_ATTRIBUTES},
	{ERRDOS,	1346,	NT_STATUS_BAD_IMPERSONATION_LEVEL},
	{ERRDOS,	1347,	NT_STATUS_CANT_OPEN_ANONYMOUS},
	{ERRDOS,	1348,	NT_STATUS_BAD_VALIDATION_CLASS},
	{ERRDOS,	1349,	NT_STATUS_BAD_TOKEN_TYPE},
	{ERRDOS,	ERRbadpipe,	NT_STATUS_BAD_MASTER_BOOT_RECORD},
	{ERRDOS,	ERRpipebusy,	NT_STATUS_INSTANCE_NOT_AVAILABLE},
	{ERRDOS,	ERRpipebusy,	NT_STATUS_PIPE_NOT_AVAILABLE},
	{ERRDOS,	ERRbadpipe,	NT_STATUS_INVALID_PIPE_STATE},
	{ERRDOS,	ERRpipebusy,	NT_STATUS_PIPE_BUSY},
	{ERRSRV,	ERRsmbcmd,	NT_STATUS_ILLEGAL_FUNCTION},
	{ERRDOS,	ERRnotconnected,	NT_STATUS_PIPE_DISCONNECTED},
	{ERRDOS,	ERRpipeclosing,	NT_STATUS_PIPE_CLOSING},
	{ERRDOS,	535,	NT_STATUS_PIPE_CONNECTED},
	{ERRDOS,	536,	NT_STATUS_PIPE_LISTENING},
	{ERRDOS,	ERRbadpipe,	NT_STATUS_INVALID_READ_MODE},
	{ERRDOS,	121,	NT_STATUS_IO_TIMEOUT},
	{ERRDOS,	38,	NT_STATUS_FILE_FORCED_CLOSED},
	{ERRDOS,	ERRbadaccess,	NT_STATUS_FILE_IS_A_DIRECTORY},
	{ERRDOS,	ERRunsup,	NT_STATUS_NOT_SUPPORTED},
	{ERRDOS,	51,	NT_STATUS_REMOTE_NOT_LISTENING},
	{ERRDOS,	52,	NT_STATUS_DUPLICATE_NAME},
	{ERRDOS,	53,	NT_STATUS_BAD_NETWORK_PATH},
	{ERRDOS,	54,	NT_STATUS_NETWORK_BUSY},
	{ERRDOS,	55,	NT_STATUS_DEVICE_DOES_NOT_EXIST},
	{ERRDOS,	56,	NT_STATUS_TOO_MANY_COMMANDS},
	{ERRDOS,	57,	NT_STATUS_ADAPTER_HARDWARE_ERROR},
	{ERRDOS,	58,	NT_STATUS_INVALID_NETWORK_RESPONSE},
	{ERRDOS,	59,	NT_STATUS_UNEXPECTED_NETWORK_ERROR},
	{ERRDOS,	60,	NT_STATUS_BAD_REMOTE_ADAPTER},
	{ERRSRV,	ERRqfull,	NT_STATUS_PRINT_QUEUE_FULL},
	{ERRSRV,	ERRqtoobig,	NT_STATUS_NO_SPOOL_SPACE},
	{ERRSRV,	ERRinvpfid,	NT_STATUS_PRINT_CANCELLED},
	{ERRSRV,	ERRinvnid,	NT_STATUS_NETWORK_NAME_DELETED},
	{ERRSRV,	ERRaccess,	NT_STATUS_NETWORK_ACCESS_DENIED},
	{ERRSRV,	ERRbadtype,	NT_STATUS_BAD_DEVICE_TYPE},
	{ERRSRV,	ERRinvnetname,	NT_STATUS_BAD_NETWORK_NAME},
	{ERRDOS,	68,	NT_STATUS_TOO_MANY_NAMES},
	{ERRSRV,	ERRtoomanyuids,	NT_STATUS_TOO_MANY_SESSIONS},
	{ERRSRV,	ERRpaused,	NT_STATUS_SHARING_PAUSED},
	{ERRSRV,	ERRmsgoff,	NT_STATUS_REQUEST_NOT_ACCEPTED},
	{ERRDOS,	72,	NT_STATUS_REDIRECTOR_PAUSED},
	{ERRDOS,	88,	NT_STATUS_NET_WRITE_FAULT},
	{ERRDOS,	ERRdiffdevice,	NT_STATUS_NOT_SAME_DEVICE},
	{ERRDOS,	ERRbadaccess,	NT_STATUS_FILE_RENAMED},
	{ERRDOS,	240,	NT_STATUS_VIRTUAL_CIRCUIT_CLOSED},
	{ERRDOS,	1350,	NT_STATUS_NO_SECURITY_ON_OBJECT},
	{ERRDOS,	ERRpipeclosing,	NT_STATUS_PIPE_EMPTY},
	{ERRDOS,	1351,	NT_STATUS_CANT_ACCESS_DOMAIN_INFO},
	{ERRDOS,	1352,	NT_STATUS_INVALID_SERVER_STATE},
	{ERRDOS,	1353,	NT_STATUS_INVALID_DOMAIN_STATE},
	{ERRDOS,	1354,	NT_STATUS_INVALID_DOMAIN_ROLE},
	{ERRDOS,	1355,	NT_STATUS_NO_SUCH_DOMAIN},
	{ERRDOS,	1356,	NT_STATUS_DOMAIN_EXISTS},
	{ERRDOS,	1357,	NT_STATUS_DOMAIN_LIMIT_EXCEEDED},
	{ERRDOS,	300,	NT_STATUS_OPLOCK_NOT_GRANTED},
	{ERRDOS,	301,	NT_STATUS_INVALID_OPLOCK_PROTOCOL},
	{ERRDOS,	1358,	NT_STATUS_INTERNAL_DB_CORRUPTION},
	{ERRDOS,	1359,	NT_STATUS_INTERNAL_ERROR},
	{ERRDOS,	1360,	NT_STATUS_GENERIC_NOT_MAPPED},
	{ERRDOS,	1361,	NT_STATUS_BAD_DESCRIPTOR_FORMAT},
	{ERRDOS,	1784,	NT_STATUS_INVALID_USER_BUFFER},
	{ERRDOS,	1362,	NT_STATUS_NOT_LOGON_PROCESS},
	{ERRDOS,	1363,	NT_STATUS_LOGON_SESSION_EXISTS},
	{ERRDOS,	ERRbadpipe,	NT_STATUS_INVALID_PARAMETER_1},
	{ERRDOS,	ERRbadpipe,	NT_STATUS_INVALID_PARAMETER_2},
	{ERRDOS,	ERRbadpipe,	NT_STATUS_INVALID_PARAMETER_3},
	{ERRDOS,	ERRbadpipe,	NT_STATUS_INVALID_PARAMETER_4},
	{ERRDOS,	ERRbadpipe,	NT_STATUS_INVALID_PARAMETER_5},
	{ERRDOS,	ERRbadpipe,	NT_STATUS_INVALID_PARAMETER_6},
	{ERRDOS,	ERRbadpipe,	NT_STATUS_INVALID_PARAMETER_7},
	{ERRDOS,	ERRbadpipe,	NT_STATUS_INVALID_PARAMETER_8},
	{ERRDOS,	ERRbadpipe,	NT_STATUS_INVALID_PARAMETER_9},
	{ERRDOS,	ERRbadpipe,	NT_STATUS_INVALID_PARAMETER_10},
	{ERRDOS,	ERRbadpipe,	NT_STATUS_INVALID_PARAMETER_11},
	{ERRDOS,	ERRbadpipe,	NT_STATUS_INVALID_PARAMETER_12},
	{ERRDOS,	ERRbadpath,	NT_STATUS_REDIRECTOR_NOT_STARTED},
	{ERRDOS,	1001,	NT_STATUS_STACK_OVERFLOW},
	{ERRDOS,	1364,	NT_STATUS_NO_SUCH_PACKAGE},
	{ERRDOS,	203,	NT_STATUS(0xc0000100)},
	{ERRDOS,	ERRremcd,	NT_STATUS_DIRECTORY_NOT_EMPTY},
	{ERRDOS,	276,	NT_STATUS_FILE_CORRUPT_ERROR},
	{ERRDOS,	267,	NT_STATUS_NOT_A_DIRECTORY},
	{ERRDOS,	1365,	NT_STATUS_BAD_LOGON_SESSION_STATE},
	{ERRDOS,	1366,	NT_STATUS_LOGON_SESSION_COLLISION},
	{ERRDOS,	206,	NT_STATUS_NAME_TOO_LONG},
	{ERRDOS,	2401,	NT_STATUS_FILES_OPEN},
	{ERRDOS,	2404,	NT_STATUS_CONNECTION_IN_USE},
	{ERRDOS,	ERRbadaccess,	NT_STATUS_PROCESS_IS_TERMINATING},
	{ERRDOS,	1367,	NT_STATUS_INVALID_LOGON_TYPE},
	{ERRDOS,	1368,	NT_STATUS_CANNOT_IMPERSONATE},
	{ERRDOS,	1056,	NT_STATUS_IMAGE_ALREADY_LOADED},
	{ERRDOS,	1444,	NT_STATUS_NO_LDT},
	{ERRDOS,	193,	NT_STATUS_INVALID_IMAGE_NE_FORMAT},
	{ERRDOS,	1369,	NT_STATUS_RXACT_INVALID_STATE},
	{ERRDOS,	1370,	NT_STATUS_RXACT_COMMIT_FAILURE},
	{ERRDOS,	1006,	NT_STATUS_MAPPED_FILE_SIZE_ZERO},
	{ERRDOS,	ERRnofids,	NT_STATUS_TOO_MANY_OPENED_FILES},
	{ERRDOS,	995,	NT_STATUS_CANCELLED},
	{ERRDOS,	ERRbadaccess,	NT_STATUS_CANNOT_DELETE},
	{ERRDOS,	1210,	NT_STATUS_INVALID_COMPUTER_NAME},
	{ERRDOS,	ERRbadaccess,	NT_STATUS_FILE_DELETED},
	{ERRDOS,	1371,	NT_STATUS_SPECIAL_ACCOUNT},
	{ERRDOS,	1372,	NT_STATUS_SPECIAL_GROUP},
	{ERRDOS,	1373,	NT_STATUS_SPECIAL_USER},
	{ERRDOS,	1374,	NT_STATUS_MEMBERS_PRIMARY_GROUP},
	{ERRDOS,	ERRbadfid,	NT_STATUS_FILE_CLOSED},
	{ERRDOS,	1375,	NT_STATUS_TOKEN_ALREADY_IN_USE},
	{ERRDOS,	1455,	NT_STATUS_COMMITMENT_LIMIT},
	{ERRDOS,	193,	NT_STATUS_INVALID_IMAGE_LE_FORMAT},
	{ERRDOS,	193,	NT_STATUS_INVALID_IMAGE_NOT_MZ},
	{ERRDOS,	193,	NT_STATUS_INVALID_IMAGE_PROTECT},
	{ERRDOS,	193,	NT_STATUS_INVALID_IMAGE_WIN_16},
	{ERRDOS,	1398,	NT_STATUS_TIME_DIFFERENCE_AT_DC},
	{ERRDOS,	126,	NT_STATUS_DLL_NOT_FOUND},
	{ERRDOS,	182,	NT_STATUS_ORDINAL_NOT_FOUND},
	{ERRDOS,	127,	NT_STATUS_ENTRYPOINT_NOT_FOUND},
	{ERRSRV,	ERRinvnid,	NT_STATUS_LOCAL_DISCONNECT},
	{ERRSRV,	ERRinvnid,	NT_STATUS_REMOTE_DISCONNECT},
	{ERRDOS,	51,	NT_STATUS_REMOTE_RESOURCES},
	{ERRDOS,	59,	NT_STATUS_LINK_FAILED},
	{ERRDOS,	59,	NT_STATUS_LINK_TIMEOUT},
	{ERRDOS,	59,	NT_STATUS_INVALID_CONNECTION},
	{ERRDOS,	59,	NT_STATUS_INVALID_ADDRESS},
	{ERRDOS,	1114,	NT_STATUS_DLL_INIT_FAILED},
	{ERRDOS,	124,	NT_STATUS_INVALID_LEVEL},
	{ERRSRV,	ERRbadpw,	NT_STATUS_WRONG_PASSWORD_CORE},
	{ERRDOS,	109,	NT_STATUS_PIPE_BROKEN},
	{ERRDOS,	1009,	NT_STATUS_REGISTRY_CORRUPT},
	{ERRDOS,	1016,	NT_STATUS_REGISTRY_IO_FAILED},
	{ERRDOS,	1005,	NT_STATUS_UNRECOGNIZED_VOLUME},
	{ERRDOS,	1118,	NT_STATUS_SERIAL_NO_DEVICE_INITED},
	{ERRDOS,	1376,	NT_STATUS_NO_SUCH_ALIAS},
	{ERRDOS,	1377,	NT_STATUS_MEMBER_NOT_IN_ALIAS},
	{ERRDOS,	1378,	NT_STATUS_MEMBER_IN_ALIAS},
	{ERRDOS,	1379,	NT_STATUS_ALIAS_EXISTS},
	{ERRDOS,	1380,	NT_STATUS_LOGON_NOT_GRANTED},
	{ERRDOS,	1381,	NT_STATUS_TOO_MANY_SECRETS},
	{ERRDOS,	1382,	NT_STATUS_SECRET_TOO_LONG},
	{ERRDOS,	1383,	NT_STATUS_INTERNAL_DB_ERROR},
	{ERRDOS,	1007,	NT_STATUS_FULLSCREEN_MODE},
	{ERRDOS,	1384,	NT_STATUS_TOO_MANY_CONTEXT_IDS},
	{ERRDOS,	1385,	NT_STATUS_LOGON_TYPE_NOT_GRANTED},
	{ERRDOS,	1017,	NT_STATUS_NOT_REGISTRY_FILE},
	{ERRDOS,	1386,	NT_STATUS_NT_CROSS_ENCRYPTION_REQUIRED},
	{ERRDOS,	1117,	NT_STATUS_FT_MISSING_MEMBER},
	{ERRDOS,	1113,	NT_STATUS_UNMAPPABLE_CHARACTER},
	{ERRDOS,	1122,	NT_STATUS_FLOPPY_ID_MARK_NOT_FOUND},
	{ERRDOS,	1123,	NT_STATUS_FLOPPY_WRONG_CYLINDER},
	{ERRDOS,	1124,	NT_STATUS_FLOPPY_UNKNOWN_ERROR},
	{ERRDOS,	1125,	NT_STATUS_FLOPPY_BAD_REGISTERS},
	{ERRDOS,	1126,	NT_STATUS_DISK_RECALIBRATE_FAILED},
	{ERRDOS,	1127,	NT_STATUS_DISK_OPERATION_FAILED},
	{ERRDOS,	1128,	NT_STATUS_DISK_RESET_FAILED},
	{ERRDOS,	1119,	NT_STATUS_SHARED_IRQ_BUSY},
	{ERRDOS,	1117,	NT_STATUS_FT_ORPHANING},
	{ERRDOS,	1105,	NT_STATUS_PARTITION_FAILURE},
	{ERRDOS,	1106,	NT_STATUS_INVALID_BLOCK_LENGTH},
	{ERRDOS,	1107,	NT_STATUS_DEVICE_NOT_PARTITIONED},
	{ERRDOS,	1108,	NT_STATUS_UNABLE_TO_LOCK_MEDIA},
	{ERRDOS,	1109,	NT_STATUS_UNABLE_TO_UNLOAD_MEDIA},
	{ERRDOS,	1129,	NT_STATUS_EOM_OVERFLOW},
	{ERRDOS,	1112,	NT_STATUS_NO_MEDIA},
	{ERRDOS,	1387,	NT_STATUS_NO_SUCH_MEMBER},
	{ERRDOS,	1388,	NT_STATUS_INVALID_MEMBER},
	{ERRDOS,	1018,	NT_STATUS_KEY_DELETED},
	{ERRDOS,	1019,	NT_STATUS_NO_LOG_SPACE},
	{ERRDOS,	1389,	NT_STATUS_TOO_MANY_SIDS},
	{ERRDOS,	1390,	NT_STATUS_LM_CROSS_ENCRYPTION_REQUIRED},
	{ERRDOS,	1020,	NT_STATUS_KEY_HAS_CHILDREN},
	{ERRDOS,	1021,	NT_STATUS_CHILD_MUST_BE_VOLATILE},
	{ERRDOS,	ERRbadpipe,	NT_STATUS_DEVICE_CONFIGURATION_ERROR},
	{ERRDOS,	1117,	NT_STATUS_DRIVER_INTERNAL_ERROR},
	{ERRDOS,	22,	NT_STATUS_INVALID_DEVICE_STATE},
	{ERRDOS,	1117,	NT_STATUS_IO_DEVICE_ERROR},
	{ERRDOS,	1117,	NT_STATUS_DEVICE_PROTOCOL_ERROR},
	{ERRDOS,	1502,	NT_STATUS_LOG_FILE_FULL},
	{ERRDOS,	19,	NT_STATUS_TOO_LATE},
	{ERRDOS,	1786,	NT_STATUS_NO_TRUST_LSA_SECRET},
	{ERRDOS,	1787,	NT_STATUS_NO_TRUST_SAM_ACCOUNT},
	{ERRDOS,	1788,	NT_STATUS_TRUSTED_DOMAIN_FAILURE},
	{ERRDOS,	1789,	NT_STATUS_TRUSTED_RELATIONSHIP_FAILURE},
	{ERRDOS,	1500,	NT_STATUS_EVENTLOG_FILE_CORRUPT},
	{ERRDOS,	1501,	NT_STATUS_EVENTLOG_CANT_START},
	{ERRDOS,	1790,	NT_STATUS_TRUST_FAILURE},
	{ERRDOS,	1792,	NT_STATUS_NETLOGON_NOT_STARTED},
	{ERRSRV,	2239,	NT_STATUS_ACCOUNT_EXPIRED},
	{ERRDOS,	1131,	NT_STATUS_POSSIBLE_DEADLOCK},
	{ERRDOS,	1219,	NT_STATUS_NETWORK_CREDENTIAL_CONFLICT},
	{ERRDOS,	1220,	NT_STATUS_REMOTE_SESSION_LIMIT},
	{ERRDOS,	1503,	NT_STATUS_EVENTLOG_FILE_CHANGED},
	{ERRDOS,	1807,	NT_STATUS_NOLOGON_INTERDOMAIN_TRUST_ACCOUNT},
	{ERRDOS,	1808,	NT_STATUS_NOLOGON_WORKSTATION_TRUST_ACCOUNT},
	{ERRDOS,	1809,	NT_STATUS_NOLOGON_SERVER_TRUST_ACCOUNT},
	{ERRDOS,	1810,	NT_STATUS_DOMAIN_TRUST_INCONSISTENT},
	{ERRDOS,	1394,	NT_STATUS_NO_USER_SESSION_KEY},
	{ERRDOS,	59,	NT_STATUS_USER_SESSION_DELETED},
	{ERRDOS,	1815,	NT_STATUS_RESOURCE_LANG_NOT_FOUND},
	{ERRDOS,	1130,	NT_STATUS_INSUFF_SERVER_RESOURCES},
	{ERRDOS,	1784,	NT_STATUS_INVALID_BUFFER_SIZE},
	{ERRDOS,	1214,	NT_STATUS_INVALID_ADDRESS_COMPONENT},
	{ERRDOS,	1214,	NT_STATUS_INVALID_ADDRESS_WILDCARD},
	{ERRDOS,	68,	NT_STATUS_TOO_MANY_ADDRESSES},
	{ERRDOS,	52,	NT_STATUS_ADDRESS_ALREADY_EXISTS},
	{ERRSRV,	ERRinvnid,	NT_STATUS_ADDRESS_CLOSED},
	{ERRSRV,	ERRinvnid,	NT_STATUS_CONNECTION_DISCONNECTED},
	{ERRSRV,	ERRinvnid,	NT_STATUS_CONNECTION_RESET},
	{ERRDOS,	68,	NT_STATUS_TOO_MANY_NODES},
	{ERRDOS,	59,	NT_STATUS_TRANSACTION_ABORTED},
	{ERRDOS,	59,	NT_STATUS_TRANSACTION_TIMED_OUT},
	{ERRDOS,	59,	NT_STATUS_TRANSACTION_NO_RELEASE},
	{ERRDOS,	59,	NT_STATUS_TRANSACTION_NO_MATCH},
	{ERRDOS,	59,	NT_STATUS_TRANSACTION_RESPONDED},
	{ERRDOS,	59,	NT_STATUS_TRANSACTION_INVALID_ID},
	{ERRDOS,	59,	NT_STATUS_TRANSACTION_INVALID_TYPE},
	{ERRDOS,	ERRunsup,	NT_STATUS_NOT_SERVER_SESSION},
	{ERRDOS,	ERRunsup,	NT_STATUS_NOT_CLIENT_SESSION},
	{ERRDOS,	6118,	NT_STATUS_NO_BROWSER_SERVERS_FOUND},
	{ERRDOS,	1132,	NT_STATUS_MAPPED_ALIGNMENT},
	{ERRDOS,	193,	NT_STATUS_IMAGE_CHECKSUM_MISMATCH},
	{ERRSRV,	2242,	NT_STATUS_PASSWORD_MUST_CHANGE},
	{ERRDOS,	1168,	NT_STATUS_NOT_FOUND},
	{ERRDOS,	554,	NT_STATUS_DUPLICATE_OBJECTID},
	{ERRDOS,	555,	NT_STATUS_OBJECTID_EXISTS},
	{ERRDOS,	1237,	NT_STATUS_RETRY},
	{ERRDOS,	1170,	NT_STATUS_PROPSET_NOT_FOUND},
	{ERRDOS,	1908,	NT_STATUS_DOMAIN_CONTROLLER_NOT_FOUND},
	{ERRDOS,	ERRnoaccess,	NT_STATUS_ACCOUNT_LOCKED_OUT},
	{ERRDOS,	ERRbadfid,	NT_STATUS_HANDLE_NOT_CLOSABLE},
	{ERRDOS,	1225,	NT_STATUS_CONNECTION_REFUSED},
	{ERRDOS,	1226,	NT_STATUS_GRACEFUL_DISCONNECT},
	{ERRDOS,	1227,	NT_STATUS_ADDRESS_ALREADY_ASSOCIATED},
	{ERRDOS,	1228,	NT_STATUS_ADDRESS_NOT_ASSOCIATED},
	{ERRDOS,	1229,	NT_STATUS_CONNECTION_INVALID},
	{ERRDOS,	1230,	NT_STATUS_CONNECTION_ACTIVE},
	{ERRDOS,	1231,	NT_STATUS_NETWORK_UNREACHABLE},
	{ERRDOS,	1232,	NT_STATUS_HOST_UNREACHABLE},
	{ERRDOS,	1233,	NT_STATUS_PROTOCOL_UNREACHABLE},
	{ERRDOS,	1234,	NT_STATUS_PORT_UNREACHABLE},
	{ERRDOS,	1235,	NT_STATUS_REQUEST_ABORTED},
	{ERRDOS,	1236,	NT_STATUS_CONNECTION_ABORTED},
	{ERRDOS,	1224,	NT_STATUS_USER_MAPPED_FILE},
	{ERRDOS,	1238,	NT_STATUS_CONNECTION_COUNT_LIMIT},
	{ERRDOS,	1239,	NT_STATUS_LOGIN_TIME_RESTRICTION},
	{ERRDOS,	1240,	NT_STATUS_LOGIN_WKSTA_RESTRICTION},
	{ERRDOS,	193,	NT_STATUS_IMAGE_MP_UP_MISMATCH},
	{ERRDOS,	1359,	NT_STATUS_LPC_REPLY_LOST},
	{ERRDOS,	1232,	NT_STATUS_PATH_NOT_COVERED},
	{ERRDOS,	1395,	NT_STATUS_LICENSE_QUOTA_EXCEEDED},
	{ERRDOS,	1058,	NT_STATUS_PLUGPLAY_NO_DEVICE},
	{ERRDOS,	182,	NT_STATUS_DRIVER_ORDINAL_NOT_FOUND},
	{ERRDOS,	127,	NT_STATUS_DRIVER_ENTRYPOINT_NOT_FOUND},
	{ERRDOS,	288,	NT_STATUS_RESOURCE_NOT_OWNED},
	{ERRDOS,	1142,	NT_STATUS_TOO_MANY_LINKS},
	{ERRDOS,	4350,	NT_STATUS_FILE_IS_OFFLINE},
	{ERRDOS,	2001,	NT_STATUS(0xc000026c)},
	{ERRDOS,	1201,	NT_STATUS(0xc000026d)},
	{ERRDOS,	21,	NT_STATUS(0xc000026e)},
	{ERRDOS,	1169,	NT_STATUS(0xc0000272)},
	{ERRDOS,	4390,	NT_STATUS(0xc0000275)},
	{ERRDOS,	4393,	NT_STATUS(0xc0000276)},
	{ERRDOS,	4394,	NT_STATUS(0xc0000277)},
	{ERRDOS,	4392,	NT_STATUS(0xc0000278)},
	{ERRDOS,	1920,	NT_STATUS(0xc0000279)},
	{ERRDOS,	1921,	NT_STATUS(0xc0000280)},
	{ERRDOS,	161,	NT_STATUS(0xc0000281)},
	{ERRDOS,	1160,	NT_STATUS(0xc0000283)},
	{ERRDOS,	1161,	NT_STATUS(0xc0000284)},
	{ERRDOS,	1162,	NT_STATUS(0xc0000285)},
	{ERRDOS,	1163,	NT_STATUS(0xc0000286)},
	{ERRDOS,	1164,	NT_STATUS(0xc0000287)},
	{ERRDOS,	ERRbadaccess,	NT_STATUS(0xc000028a)},
	{ERRDOS,	ERRbadaccess,	NT_STATUS(0xc000028b)},
	{ERRDOS,	ERRbadaccess,	NT_STATUS(0xc000028d)},
	{ERRDOS,	ERRbadaccess,	NT_STATUS(0xc000028e)},
	{ERRDOS,	ERRbadaccess,	NT_STATUS(0xc000028f)},
	{ERRDOS,	ERRbadaccess,	NT_STATUS(0xc0000290)},
	{ERRDOS,	6007,	NT_STATUS(0xc0000291)},
	{ERRDOS,	6008,	NT_STATUS(0xc0000292)},
	{ERRDOS,	6002,	NT_STATUS(0xc0000293)},
	{ERRDOS,	4200,	NT_STATUS(0xc0000295)},
	{ERRDOS,	4201,	NT_STATUS(0xc0000296)},
	{ERRDOS,	4202,	NT_STATUS(0xc0000297)},
	{ERRDOS,	4203,	NT_STATUS(0xc0000298)},
	{ERRDOS,	8218,	NT_STATUS(0xc0000299)},
	{ERRDOS,	8219,	NT_STATUS(0xc000029a)},
	{ERRDOS,	8220,	NT_STATUS(0xc000029b)},
	{ERRSRV,	ERRsmbcmd,	NT_STATUS(0xc000029c)},
	{ERRDOS,	4351,	NT_STATUS(0xc000029d)},
	{ERRDOS,	4352,	NT_STATUS(0xc000029e)},
	{ERRDOS,	1172,	NT_STATUS(0xc000029f)},
	{ERRDOS,	8202,	NT_STATUS(0xc00002a1)},
	{ERRDOS,	8203,	NT_STATUS(0xc00002a2)},
	{ERRDOS,	8204,	NT_STATUS(0xc00002a3)},
	{ERRDOS,	8205,	NT_STATUS(0xc00002a4)},
	{ERRDOS,	8206,	NT_STATUS(0xc00002a5)},
	{ERRDOS,	8207,	NT_STATUS(0xc00002a6)},
	{ERRDOS,	8208,	NT_STATUS(0xc00002a7)},
	{ERRDOS,	8209,	NT_STATUS(0xc00002a8)},
	{ERRDOS,	8210,	NT_STATUS(0xc00002a9)},
	{ERRDOS,	8211,	NT_STATUS(0xc00002aa)},
	{ERRDOS,	8212,	NT_STATUS(0xc00002ab)},
	{ERRDOS,	8213,	NT_STATUS(0xc00002ac)},
	{ERRDOS,	8214,	NT_STATUS(0xc00002ad)},
	{ERRDOS,	8215,	NT_STATUS(0xc00002ae)},
	{ERRDOS,	8216,	NT_STATUS(0xc00002af)},
	{ERRDOS,	8217,	NT_STATUS(0xc00002b0)},
	{ERRDOS,	8478,	NT_STATUS(0xc00002b1)},
	{ERRDOS,	4391,	NT_STATUS(0xc00002b2)},
	{ERRDOS,	1617,	NT_STATUS(0xc00002b6)},
	{ERRDOS,	1178,	NT_STATUS(0xc00002b7)},
	{ERRDOS,	1179,	NT_STATUS(0xc00002b8)},
	{ERRDOS,	8228,	NT_STATUS(0xc00002c1)},
	{ERRDOS,	1397,	NT_STATUS(0xc00002c3)},
	{ERRDOS,	998,	NT_STATUS(0xc00002c5)},
	{ERRDOS,	4213,	NT_STATUS(0xc00002c6)},
	{ERRDOS,	4214,	NT_STATUS(0xc00002c7)},
	{ERRDOS,	4328,	NT_STATUS(0xc00002ca)},
	{ERRDOS,	8504,	NT_STATUS(0xc00002cb)},
	{ERRDOS,	1251,	NT_STATUS(0xc00002cc)},
	{ERRDOS,	8505,	NT_STATUS(0xc00002cd)},
	{ERRDOS,	1181,	NT_STATUS(0xc00002cf)},
	{ERRDOS,	8506,	NT_STATUS(0xc00002d0)},
	{ERRDOS,	8513,	NT_STATUS(0xc00002d4)},
	{ERRDOS,	8514,	NT_STATUS(0xc00002d5)},
	{ERRDOS,	8515,	NT_STATUS(0xc00002d6)},
	{ERRDOS,	8516,	NT_STATUS(0xc00002d7)},
	{ERRDOS,	8517,	NT_STATUS(0xc00002d8)},
	{ERRDOS,	8518,	NT_STATUS(0xc00002d9)},
	{ERRDOS,	8519,	NT_STATUS(0xc00002da)},
	{ERRDOS,	8520,	NT_STATUS(0xc00002db)},
	{ERRDOS,	8521,	NT_STATUS(0xc00002dc)},
	{ERRDOS,	ERRunsup,	NT_STATUS(0xc00002dd)},
	{ERRDOS,	8529,	NT_STATUS(0xc00002df)},
	{ERRDOS,	8530,	NT_STATUS(0xc00002e0)},
	{ERRDOS,	8531,	NT_STATUS(0xc00002e1)},
	{ERRDOS,	8532,	NT_STATUS(0xc00002e2)},
	{ERRDOS,	8541,	NT_STATUS(0xc00002e3)},
	{ERRDOS,	8547,	NT_STATUS(0xc00002e4)},
	{ERRDOS,	8548,	NT_STATUS(0xc00002e5)},
	{ERRDOS,	8549,	NT_STATUS(0xc00002e6)},
	{ERRDOS,	8557,	NT_STATUS(0xc00002e7)},
	{ERRDOS,	1115,	NT_STATUS(0xc00002fe)},
	{ERRDOS,	1255,	NT_STATUS(0xc00002ff)},
	{ERRDOS,	1254,	NT_STATUS(0xc0000300)},
	{ERRDOS,	ERRbadfunc,	NT_STATUS(0x80000001)},
	{ERRDOS,	998,	NT_STATUS(0x80000002)},
	{ERRDOS,	ERRbadpath,	NT_STATUS(0x80000003)},
	{ERRDOS,	ERRnofids,	NT_STATUS(0x80000004)},
	{ERRDOS,	111,	NT_STATUS(0x80000005)},
	{ERRDOS,	ERRnofiles,	NT_STATUS(0x80000006)},
	{ERRDOS,	1391,	NT_STATUS(0x8000000b)},
	{ERRDOS,	299,	NT_STATUS(0x8000000d)},
	{ERRDOS,	28,	NT_STATUS(0x8000000e)},
	{ERRDOS,	21,	NT_STATUS(0x8000000f)},
	{ERRDOS,	21,	NT_STATUS(0x80000010)},
	{ERRDOS,	170,	NT_STATUS(0x80000011)},
	{ERRDOS,	259,	NT_STATUS(0x80000012)},
	{ERRDOS,	254,	NT_STATUS(0x80000013)},
	{ERRDOS,	275,	NT_STATUS(0x80000014)},
	{ERRDOS,	275,	NT_STATUS(0x80000015)},
	{ERRDOS,	1110,	NT_STATUS(0x80000016)},
	{ERRDOS,	259,	NT_STATUS_UNABLE_TO_FREE_VM},
	{ERRDOS,	1101,	NT_STATUS(0x8000001b)},
	{ERRDOS,	1110,	NT_STATUS(0x8000001c)},
	{ERRDOS,	1111,	NT_STATUS(0x8000001d)},
	{ERRDOS,	1100,	NT_STATUS(0x8000001e)},
	{ERRDOS,	1102,	NT_STATUS(0x8000001f)},
	{ERRDOS,	1103,	NT_STATUS(0x80000021)},
	{ERRDOS,	1104,	NT_STATUS(0x80000022)},
	{ERRDOS,	2402,	NT_STATUS(0x80000025)},
	{ERRDOS,	1165,	NT_STATUS(0x80000288)},
	{ERRDOS,	1166,	NT_STATUS(0x80000289)},
};


/* dos -> nt status error map */
static struct {
	uint8 dos_class;
	uint32 dos_code;
	NTSTATUS ntstatus;
} dos_to_ntstatus_map[] = {
	{ERRDOS,	ERRbadfunc,	NT_STATUS_NOT_IMPLEMENTED},
	{ERRDOS,	ERRbadfile,	NT_STATUS_NO_SUCH_FILE},
	{ERRDOS,	ERRbadpath,	NT_STATUS_OBJECT_PATH_NOT_FOUND},
	{ERRDOS,	ERRnofids,	NT_STATUS_TOO_MANY_OPENED_FILES},
	{ERRDOS,	ERRnoaccess,	NT_STATUS_ACCESS_DENIED},
	{ERRDOS,	ERRbadfid,	NT_STATUS_INVALID_HANDLE},
	{ERRDOS,	ERRnomem,	NT_STATUS_INSUFFICIENT_RESOURCES},
	{ERRDOS,	ERRbadaccess,	NT_STATUS_INVALID_LOCK_SEQUENCE},
	{ERRDOS,	ERRbaddata,	NT_STATUS_DATA_ERROR},
	{ERRDOS,	14,	NT_STATUS_SECTION_NOT_EXTENDED},
	{ERRDOS,	ERRremcd,	NT_STATUS_DIRECTORY_NOT_EMPTY},
	{ERRDOS,	ERRdiffdevice,	NT_STATUS_NOT_SAME_DEVICE},
	{ERRDOS,	ERRnofiles,	NT_STATUS(0x80000006)},
	{ERRDOS,	19,	NT_STATUS_MEDIA_WRITE_PROTECTED},
	{ERRDOS,	21,	NT_STATUS_NO_MEDIA_IN_DEVICE},
	{ERRDOS,	22,	NT_STATUS_INVALID_DEVICE_STATE},
	{ERRDOS,	23,	NT_STATUS_DATA_ERROR},
	{ERRDOS,	24,	NT_STATUS_DATA_ERROR},
	{ERRDOS,	26,	NT_STATUS_DISK_CORRUPT_ERROR},
	{ERRDOS,	27,	NT_STATUS_NONEXISTENT_SECTOR},
	{ERRDOS,	28,	NT_STATUS(0x8000000e)},
	{ERRDOS,	31,	NT_STATUS_UNSUCCESSFUL},
	{ERRDOS,	ERRbadshare,	NT_STATUS_SHARING_VIOLATION},
	{ERRDOS,	ERRlock,	NT_STATUS_FILE_LOCK_CONFLICT},
	{ERRDOS,	34,	NT_STATUS_WRONG_VOLUME},
	{ERRDOS,	38,	NT_STATUS_END_OF_FILE},
	{ERRDOS,	ERRunsup,	NT_STATUS_CTL_FILE_NOT_SUPPORTED},
	{ERRDOS,	51,	NT_STATUS_REMOTE_NOT_LISTENING},
	{ERRDOS,	52,	NT_STATUS_DUPLICATE_NAME},
	{ERRDOS,	53,	NT_STATUS_BAD_NETWORK_PATH},
	{ERRDOS,	54,	NT_STATUS_NETWORK_BUSY},
	{ERRDOS,	55,	NT_STATUS_DEVICE_DOES_NOT_EXIST},
	{ERRDOS,	56,	NT_STATUS_TOO_MANY_COMMANDS},
	{ERRDOS,	57,	NT_STATUS_ADAPTER_HARDWARE_ERROR},
	{ERRDOS,	58,	NT_STATUS_INVALID_NETWORK_RESPONSE},
	{ERRDOS,	59,	NT_STATUS_UNEXPECTED_NETWORK_ERROR},
	{ERRDOS,	60,	NT_STATUS_BAD_REMOTE_ADAPTER},
	{ERRDOS,	61,	NT_STATUS_PRINT_QUEUE_FULL},
	{ERRDOS,	62,	NT_STATUS_NO_SPOOL_SPACE},
	{ERRDOS,	63,	NT_STATUS_PRINT_CANCELLED},
	{ERRDOS,	64,	NT_STATUS_NETWORK_NAME_DELETED},
	{ERRDOS,	65,	NT_STATUS_NETWORK_ACCESS_DENIED},
	{ERRDOS,	66,	NT_STATUS_BAD_DEVICE_TYPE},
	{ERRDOS,	ERRnosuchshare,	NT_STATUS_BAD_NETWORK_NAME},
	{ERRDOS,	68,	NT_STATUS_TOO_MANY_GUIDS_REQUESTED},
	{ERRDOS,	69,	NT_STATUS_TOO_MANY_SESSIONS},
	{ERRDOS,	70,	NT_STATUS_SHARING_PAUSED},
	{ERRDOS,	71,	NT_STATUS_REQUEST_NOT_ACCEPTED},
	{ERRDOS,	72,	NT_STATUS_REDIRECTOR_PAUSED},
	{ERRDOS,	ERRfilexists,	NT_STATUS_OBJECT_NAME_COLLISION},
	{ERRDOS,	86,	NT_STATUS_WRONG_PASSWORD},
	{ERRDOS,	87,	NT_STATUS_INVALID_INFO_CLASS},
	{ERRDOS,	88,	NT_STATUS_NET_WRITE_FAULT},
	{ERRDOS,	109,	NT_STATUS_PIPE_BROKEN},
	{ERRDOS,	111,	STATUS_MORE_ENTRIES},
	{ERRDOS,	112,	NT_STATUS_DISK_FULL},
	{ERRDOS,	121,	NT_STATUS_IO_TIMEOUT},
	{ERRDOS,	122,	NT_STATUS_BUFFER_TOO_SMALL},
	{ERRDOS,	ERRinvalidname,	NT_STATUS_OBJECT_NAME_INVALID},
	{ERRDOS,	124,	NT_STATUS_INVALID_LEVEL},
	{ERRDOS,	126,	NT_STATUS_DLL_NOT_FOUND},
	{ERRDOS,	127,	NT_STATUS_PROCEDURE_NOT_FOUND},
	{ERRDOS,	145,	NT_STATUS_DIRECTORY_NOT_EMPTY},
	{ERRDOS,	154,	NT_STATUS_INVALID_VOLUME_LABEL},
	{ERRDOS,	156,	NT_STATUS_SUSPEND_COUNT_EXCEEDED},
	{ERRDOS,	158,	NT_STATUS_NOT_LOCKED},
	{ERRDOS,	161,	NT_STATUS_OBJECT_PATH_INVALID},
	{ERRDOS,	170,	NT_STATUS(0x80000011)},
	{ERRDOS,	182,	NT_STATUS_ORDINAL_NOT_FOUND},
	{ERRDOS,	183,	NT_STATUS_OBJECT_NAME_COLLISION},
	{ERRDOS,	193,	NT_STATUS_BAD_INITIAL_PC},
	{ERRDOS,	203,	NT_STATUS(0xc0000100)},
	{ERRDOS,	206,	NT_STATUS_NAME_TOO_LONG},
	{ERRDOS,	ERRbadpipe,	NT_STATUS_INVALID_INFO_CLASS},
	{ERRDOS,	ERRpipebusy,	NT_STATUS_INSTANCE_NOT_AVAILABLE},
	{ERRDOS,	ERRpipeclosing,	NT_STATUS_PIPE_CLOSING},
	{ERRDOS,	ERRnotconnected,	NT_STATUS_PIPE_DISCONNECTED},
	{ERRDOS,	ERRmoredata,	STATUS_MORE_ENTRIES},
	{ERRDOS,	240,	NT_STATUS_VIRTUAL_CIRCUIT_CLOSED},
	{ERRDOS,	254,	NT_STATUS(0x80000013)},
	{ERRDOS,	255,	NT_STATUS_EA_TOO_LARGE},
	{ERRDOS,	259,	NT_STATUS_GUIDS_EXHAUSTED},
	{ERRDOS,	267,	NT_STATUS_NOT_A_DIRECTORY},
	{ERRDOS,	275,	NT_STATUS_EA_TOO_LARGE},
	{ERRDOS,	276,	NT_STATUS_NONEXISTENT_EA_ENTRY},
	{ERRDOS,	277,	NT_STATUS_NONEXISTENT_EA_ENTRY},
	{ERRDOS,	278,	NT_STATUS_NONEXISTENT_EA_ENTRY},
	{ERRDOS,	282,	NT_STATUS_EAS_NOT_SUPPORTED},
	{ERRDOS,	288,	NT_STATUS_MUTANT_NOT_OWNED},
	{ERRDOS,	298,	NT_STATUS_SEMAPHORE_LIMIT_EXCEEDED},
	{ERRDOS,	299,	NT_STATUS(0x8000000d)},
	{ERRDOS,	300,	NT_STATUS_OPLOCK_NOT_GRANTED},
	{ERRDOS,	301,	NT_STATUS_INVALID_OPLOCK_PROTOCOL},
	{ERRDOS,	487,	NT_STATUS_CONFLICTING_ADDRESSES},
	{ERRDOS,	534,	NT_STATUS_INTEGER_OVERFLOW},
	{ERRDOS,	535,	NT_STATUS_PIPE_CONNECTED},
	{ERRDOS,	536,	NT_STATUS_PIPE_LISTENING},
	{ERRDOS,	995,	NT_STATUS_CANCELLED},
	{ERRDOS,	997,	NT_STATUS(0x00000103)},
	{ERRDOS,	998,	NT_STATUS_ACCESS_VIOLATION},
	{ERRDOS,	999,	NT_STATUS_IN_PAGE_ERROR},
	{ERRDOS,	1001,	NT_STATUS_BAD_INITIAL_STACK},
	{ERRDOS,	1005,	NT_STATUS_UNRECOGNIZED_VOLUME},
	{ERRDOS,	1006,	NT_STATUS_FILE_INVALID},
	{ERRDOS,	1007,	NT_STATUS_FULLSCREEN_MODE},
	{ERRDOS,	1008,	NT_STATUS_NO_TOKEN},
	{ERRDOS,	1009,	NT_STATUS_REGISTRY_CORRUPT},
	{ERRDOS,	1016,	NT_STATUS_REGISTRY_IO_FAILED},
	{ERRDOS,	1017,	NT_STATUS_NOT_REGISTRY_FILE},
	{ERRDOS,	1018,	NT_STATUS_KEY_DELETED},
	{ERRDOS,	1019,	NT_STATUS_NO_LOG_SPACE},
	{ERRDOS,	1020,	NT_STATUS_KEY_HAS_CHILDREN},
	{ERRDOS,	1021,	NT_STATUS_CHILD_MUST_BE_VOLATILE},
	{ERRDOS,	1022,	NT_STATUS(0x0000010c)},
	{ERRSRV,	ERRbadpw,	NT_STATUS_WRONG_PASSWORD},
	{ERRSRV,	ERRbadtype,	NT_STATUS_BAD_DEVICE_TYPE},
	{ERRSRV,	ERRaccess,	NT_STATUS_NETWORK_ACCESS_DENIED},
	{ERRSRV,	ERRinvnid,	NT_STATUS_NETWORK_NAME_DELETED},
	{ERRSRV,	ERRinvnetname,	NT_STATUS_BAD_NETWORK_NAME},
	{ERRSRV,	ERRinvdevice,	NT_STATUS_BAD_DEVICE_TYPE},
	{ERRSRV,	ERRqfull,	NT_STATUS_PRINT_QUEUE_FULL},
	{ERRSRV,	ERRqtoobig,	NT_STATUS_NO_SPOOL_SPACE},
	{ERRSRV,	ERRinvpfid,	NT_STATUS_PRINT_CANCELLED},
	{ERRSRV,	ERRsmbcmd,	NT_STATUS_NOT_IMPLEMENTED},
	{ERRSRV,	ERRbadpermits,	NT_STATUS_NETWORK_ACCESS_DENIED},
	{ERRSRV,	ERRpaused,	NT_STATUS_SHARING_PAUSED},
	{ERRSRV,	ERRmsgoff,	NT_STATUS_REQUEST_NOT_ACCEPTED},
	{ERRSRV,	ERRnoroom,	NT_STATUS_DISK_FULL},
	{ERRSRV,	ERRnoresource,	NT_STATUS_REQUEST_NOT_ACCEPTED},
	{ERRSRV,	ERRtoomanyuids,	NT_STATUS_TOO_MANY_SESSIONS},
	{ERRSRV,	123,	NT_STATUS_OBJECT_NAME_INVALID},
	{ERRSRV,	206,	NT_STATUS_OBJECT_NAME_INVALID},
	{ERRHRD,	1,	NT_STATUS_NOT_IMPLEMENTED},
	{ERRHRD,	2,	NT_STATUS_NO_SUCH_DEVICE},
	{ERRHRD,	3,	NT_STATUS_OBJECT_PATH_NOT_FOUND},
	{ERRHRD,	4,	NT_STATUS_TOO_MANY_OPENED_FILES},
	{ERRHRD,	5,	NT_STATUS_INVALID_LOCK_SEQUENCE},
	{ERRHRD,	6,	NT_STATUS_INVALID_HANDLE},
	{ERRHRD,	8,	NT_STATUS_INSUFFICIENT_RESOURCES},
	{ERRHRD,	12,	NT_STATUS_INVALID_LOCK_SEQUENCE},
	{ERRHRD,	13,	NT_STATUS_DATA_ERROR},
	{ERRHRD,	14,	NT_STATUS_SECTION_NOT_EXTENDED},
	{ERRHRD,	16,	NT_STATUS_DIRECTORY_NOT_EMPTY},
	{ERRHRD,	17,	NT_STATUS_NOT_SAME_DEVICE},
	{ERRHRD,	18,	NT_STATUS(0x80000006)},
	{ERRHRD,	ERRnowrite,	NT_STATUS_MEDIA_WRITE_PROTECTED},
	{ERRHRD,	ERRnotready,	NT_STATUS_NO_MEDIA_IN_DEVICE},
	{ERRHRD,	ERRbadcmd,	NT_STATUS_INVALID_DEVICE_STATE},
	{ERRHRD,	ERRdata,	NT_STATUS_DATA_ERROR},
	{ERRHRD,	ERRbadreq,	NT_STATUS_DATA_ERROR},
	{ERRHRD,	ERRbadmedia,	NT_STATUS_DISK_CORRUPT_ERROR},
	{ERRHRD,	ERRbadsector,	NT_STATUS_NONEXISTENT_SECTOR},
	{ERRHRD,	ERRnopaper,	NT_STATUS(0x8000000e)},
	{ERRHRD,	ERRgeneral,	NT_STATUS_UNSUCCESSFUL},
	{ERRHRD,	ERRbadshare,	NT_STATUS_SHARING_VIOLATION},
	{ERRHRD,	ERRlock,	NT_STATUS_FILE_LOCK_CONFLICT},
	{ERRHRD,	ERRwrongdisk,	NT_STATUS_WRONG_VOLUME},
	{ERRHRD,	38,	NT_STATUS_END_OF_FILE},
	{ERRHRD,	50,	NT_STATUS_CTL_FILE_NOT_SUPPORTED},
	{ERRHRD,	51,	NT_STATUS_REMOTE_NOT_LISTENING},
	{ERRHRD,	52,	NT_STATUS_DUPLICATE_NAME},
	{ERRHRD,	53,	NT_STATUS_BAD_NETWORK_PATH},
	{ERRHRD,	54,	NT_STATUS_NETWORK_BUSY},
	{ERRHRD,	55,	NT_STATUS_DEVICE_DOES_NOT_EXIST},
	{ERRHRD,	56,	NT_STATUS_TOO_MANY_COMMANDS},
	{ERRHRD,	57,	NT_STATUS_ADAPTER_HARDWARE_ERROR},
	{ERRHRD,	58,	NT_STATUS_INVALID_NETWORK_RESPONSE},
	{ERRHRD,	59,	NT_STATUS_UNEXPECTED_NETWORK_ERROR},
	{ERRHRD,	60,	NT_STATUS_BAD_REMOTE_ADAPTER},
	{ERRHRD,	61,	NT_STATUS_PRINT_QUEUE_FULL},
	{ERRHRD,	62,	NT_STATUS_NO_SPOOL_SPACE},
	{ERRHRD,	63,	NT_STATUS_PRINT_CANCELLED},
	{ERRHRD,	64,	NT_STATUS_NETWORK_NAME_DELETED},
	{ERRHRD,	65,	NT_STATUS_NETWORK_ACCESS_DENIED},
	{ERRHRD,	66,	NT_STATUS_BAD_DEVICE_TYPE},
	{ERRHRD,	67,	NT_STATUS_BAD_NETWORK_NAME},
	{ERRHRD,	68,	NT_STATUS_TOO_MANY_GUIDS_REQUESTED},
	{ERRHRD,	69,	NT_STATUS_TOO_MANY_SESSIONS},
	{ERRHRD,	70,	NT_STATUS_SHARING_PAUSED},
	{ERRHRD,	71,	NT_STATUS_REQUEST_NOT_ACCEPTED},
	{ERRHRD,	72,	NT_STATUS_REDIRECTOR_PAUSED},
	{ERRHRD,	80,	NT_STATUS_OBJECT_NAME_COLLISION},
	{ERRHRD,	86,	NT_STATUS_WRONG_PASSWORD},
	{ERRHRD,	87,	NT_STATUS_INVALID_INFO_CLASS},
	{ERRHRD,	88,	NT_STATUS_NET_WRITE_FAULT},
	{ERRHRD,	109,	NT_STATUS_PIPE_BROKEN},
	{ERRHRD,	111,	STATUS_MORE_ENTRIES},
	{ERRHRD,	112,	NT_STATUS_DISK_FULL},
	{ERRHRD,	121,	NT_STATUS_IO_TIMEOUT},
	{ERRHRD,	122,	NT_STATUS_BUFFER_TOO_SMALL},
	{ERRHRD,	123,	NT_STATUS_OBJECT_NAME_INVALID},
	{ERRHRD,	124,	NT_STATUS_INVALID_LEVEL},
	{ERRHRD,	126,	NT_STATUS_DLL_NOT_FOUND},
	{ERRHRD,	127,	NT_STATUS_PROCEDURE_NOT_FOUND},
	{ERRHRD,	145,	NT_STATUS_DIRECTORY_NOT_EMPTY},
	{ERRHRD,	154,	NT_STATUS_INVALID_VOLUME_LABEL},
	{ERRHRD,	156,	NT_STATUS_SUSPEND_COUNT_EXCEEDED},
	{ERRHRD,	158,	NT_STATUS_NOT_LOCKED},
	{ERRHRD,	161,	NT_STATUS_OBJECT_PATH_INVALID},
	{ERRHRD,	170,	NT_STATUS(0x80000011)},
	{ERRHRD,	182,	NT_STATUS_ORDINAL_NOT_FOUND},
	{ERRHRD,	183,	NT_STATUS_OBJECT_NAME_COLLISION},
	{ERRHRD,	193,	NT_STATUS_BAD_INITIAL_PC},
	{ERRHRD,	203,	NT_STATUS(0xc0000100)},
	{ERRHRD,	206,	NT_STATUS_NAME_TOO_LONG},
	{ERRHRD,	230,	NT_STATUS_INVALID_INFO_CLASS},
	{ERRHRD,	231,	NT_STATUS_INSTANCE_NOT_AVAILABLE},
	{ERRHRD,	232,	NT_STATUS_PIPE_CLOSING},
	{ERRHRD,	233,	NT_STATUS_PIPE_DISCONNECTED},
	{ERRHRD,	234,	STATUS_MORE_ENTRIES},
	{ERRHRD,	240,	NT_STATUS_VIRTUAL_CIRCUIT_CLOSED},
	{ERRHRD,	254,	NT_STATUS(0x80000013)},
	{ERRHRD,	255,	NT_STATUS_EA_TOO_LARGE},
	{ERRHRD,	259,	NT_STATUS_GUIDS_EXHAUSTED},
	{ERRHRD,	267,	NT_STATUS_NOT_A_DIRECTORY},
	{ERRHRD,	275,	NT_STATUS_EA_TOO_LARGE},
	{ERRHRD,	276,	NT_STATUS_NONEXISTENT_EA_ENTRY},
	{ERRHRD,	277,	NT_STATUS_NONEXISTENT_EA_ENTRY},
	{ERRHRD,	278,	NT_STATUS_NONEXISTENT_EA_ENTRY},
	{ERRHRD,	282,	NT_STATUS_EAS_NOT_SUPPORTED},
	{ERRHRD,	288,	NT_STATUS_MUTANT_NOT_OWNED},
	{ERRHRD,	298,	NT_STATUS_SEMAPHORE_LIMIT_EXCEEDED},
	{ERRHRD,	299,	NT_STATUS(0x8000000d)},
	{ERRHRD,	300,	NT_STATUS_OPLOCK_NOT_GRANTED},
	{ERRHRD,	301,	NT_STATUS_INVALID_OPLOCK_PROTOCOL},
	{ERRHRD,	487,	NT_STATUS_CONFLICTING_ADDRESSES},
	{ERRHRD,	534,	NT_STATUS_INTEGER_OVERFLOW},
	{ERRHRD,	535,	NT_STATUS_PIPE_CONNECTED},
	{ERRHRD,	536,	NT_STATUS_PIPE_LISTENING},
	{ERRHRD,	995,	NT_STATUS_CANCELLED},
	{ERRHRD,	997,	NT_STATUS(0x00000103)},
	{ERRHRD,	998,	NT_STATUS_ACCESS_VIOLATION},
	{ERRHRD,	999,	NT_STATUS_IN_PAGE_ERROR},
	{ERRHRD,	1001,	NT_STATUS_BAD_INITIAL_STACK},
	{ERRHRD,	1005,	NT_STATUS_UNRECOGNIZED_VOLUME},
	{ERRHRD,	1006,	NT_STATUS_FILE_INVALID},
	{ERRHRD,	1007,	NT_STATUS_FULLSCREEN_MODE},
	{ERRHRD,	1008,	NT_STATUS_NO_TOKEN},
	{ERRHRD,	1009,	NT_STATUS_REGISTRY_CORRUPT},
	{ERRHRD,	1016,	NT_STATUS_REGISTRY_IO_FAILED},
	{ERRHRD,	1017,	NT_STATUS_NOT_REGISTRY_FILE},
	{ERRHRD,	1018,	NT_STATUS_KEY_DELETED},
	{ERRHRD,	1019,	NT_STATUS_NO_LOG_SPACE},
	{ERRHRD,	1020,	NT_STATUS_KEY_HAS_CHILDREN},
	{ERRHRD,	1021,	NT_STATUS_CHILD_MUST_BE_VOLATILE},
	{ERRHRD,	1022,	NT_STATUS(0x0000010c)},
};

/* errmap NTSTATUS->Win32 */
static struct {
	NTSTATUS ntstatus;
	WERROR werror;
} ntstatus_to_werror_map[] = {
	{NT_STATUS(0x103), W_ERROR(0x3e5)},
	{NT_STATUS(0x105), W_ERROR(0xea)},
	{NT_STATUS(0x106), W_ERROR(0x514)},
	{NT_STATUS(0x107), W_ERROR(0x515)},
	{NT_STATUS(0x10c), W_ERROR(0x3fe)},
	{NT_STATUS(0x10d), W_ERROR(0x516)},
	{NT_STATUS(0x121), W_ERROR(0x2009)},
	{NT_STATUS(0xc0000001), W_ERROR(0x1f)},
	{NT_STATUS(0xc0000002), W_ERROR(0x1)},
	{NT_STATUS(0xc0000003), W_ERROR(0x57)},
	{NT_STATUS(0xc0000004), W_ERROR(0x18)},
	{NT_STATUS(0xc0000005), W_ERROR(0x3e6)},
	{NT_STATUS(0xc0000006), W_ERROR(0x3e7)},
	{NT_STATUS(0xc0000007), W_ERROR(0x5ae)},
	{NT_STATUS(0xc0000008), W_ERROR(0x6)},
	{NT_STATUS(0xc0000009), W_ERROR(0x3e9)},
	{NT_STATUS(0xc000000a), W_ERROR(0xc1)},
	{NT_STATUS(0xc000000b), W_ERROR(0x57)},
	{NT_STATUS(0xc000000d), W_ERROR(0x57)},
	{NT_STATUS(0xc000000e), W_ERROR(0x2)},
	{NT_STATUS(0xc000000f), W_ERROR(0x2)},
	{NT_STATUS(0xc0000010), W_ERROR(0x1)},
	{NT_STATUS(0xc0000011), W_ERROR(0x26)},
	{NT_STATUS(0xc0000012), W_ERROR(0x22)},
	{NT_STATUS(0xc0000013), W_ERROR(0x15)},
	{NT_STATUS(0xc0000014), W_ERROR(0x6f9)},
	{NT_STATUS(0xc0000015), W_ERROR(0x1b)},
	{NT_STATUS(0xc0000016), W_ERROR(0xea)},
	{NT_STATUS(0xc0000017), W_ERROR(0x8)},
	{NT_STATUS(0xc0000018), W_ERROR(0x1e7)},
	{NT_STATUS(0xc0000019), W_ERROR(0x1e7)},
	{NT_STATUS(0xc000001a), W_ERROR(0x57)},
	{NT_STATUS(0xc000001b), W_ERROR(0x57)},
	{NT_STATUS(0xc000001c), W_ERROR(0x1)},
	{NT_STATUS(0xc000001d), W_ERROR(0xc000001d)},
	{NT_STATUS(0xc000001e), W_ERROR(0x5)},
	{NT_STATUS(0xc000001f), W_ERROR(0x5)},
	{NT_STATUS(0xc0000020), W_ERROR(0xc1)},
	{NT_STATUS(0xc0000021), W_ERROR(0x5)},
	{NT_STATUS(0xc0000022), W_ERROR(0x5)},
	{NT_STATUS(0xc0000023), W_ERROR(0x7a)},
	{NT_STATUS(0xc0000024), W_ERROR(0x6)},
	{NT_STATUS(0xc0000025), W_ERROR(0xc0000025)},
	{NT_STATUS(0xc0000026), W_ERROR(0xc0000026)},
	{NT_STATUS(0xc000002a), W_ERROR(0x9e)},
	{NT_STATUS(0xc000002b), W_ERROR(0xc000002b)},
	{NT_STATUS(0xc000002c), W_ERROR(0x1e7)},
	{NT_STATUS(0xc000002d), W_ERROR(0x1e7)},
	{NT_STATUS(0xc0000030), W_ERROR(0x57)},
	{NT_STATUS(0xc0000032), W_ERROR(0x571)},
	{NT_STATUS(0xc0000033), W_ERROR(0x7b)},
	{NT_STATUS(0xc0000034), W_ERROR(0x2)},
	{NT_STATUS(0xc0000035), W_ERROR(0xb7)},
	{NT_STATUS(0xc0000037), W_ERROR(0x6)},
	{NT_STATUS(0xc0000039), W_ERROR(0xa1)},
	{NT_STATUS(0xc000003a), W_ERROR(0x3)},
	{NT_STATUS(0xc000003b), W_ERROR(0xa1)},
	{NT_STATUS(0xc000003c), W_ERROR(0x45d)},
	{NT_STATUS(0xc000003d), W_ERROR(0x45d)},
	{NT_STATUS(0xc000003e), W_ERROR(0x17)},
	{NT_STATUS(0xc000003f), W_ERROR(0x17)},
	{NT_STATUS(0xc0000040), W_ERROR(0x8)},
	{NT_STATUS(0xc0000041), W_ERROR(0x5)},
	{NT_STATUS(0xc0000042), W_ERROR(0x6)},
	{NT_STATUS(0xc0000043), W_ERROR(0x20)},
	{NT_STATUS(0xc0000044), W_ERROR(0x718)},
	{NT_STATUS(0xc0000045), W_ERROR(0x57)},
	{NT_STATUS(0xc0000046), W_ERROR(0x120)},
	{NT_STATUS(0xc0000047), W_ERROR(0x12a)},
	{NT_STATUS(0xc0000048), W_ERROR(0x57)},
	{NT_STATUS(0xc0000049), W_ERROR(0x57)},
	{NT_STATUS(0xc000004a), W_ERROR(0x9c)},
	{NT_STATUS(0xc000004b), W_ERROR(0x5)},
	{NT_STATUS(0xc000004c), W_ERROR(0x57)},
	{NT_STATUS(0xc000004d), W_ERROR(0x57)},
	{NT_STATUS(0xc000004e), W_ERROR(0x57)},
	{NT_STATUS(0xc000004f), W_ERROR(0x11a)},
	{NT_STATUS(0xc0000050), W_ERROR(0xff)},
	{NT_STATUS(0xc0000051), W_ERROR(0x570)},
	{NT_STATUS(0xc0000052), W_ERROR(0x570)},
	{NT_STATUS(0xc0000053), W_ERROR(0x570)},
	{NT_STATUS(0xc0000054), W_ERROR(0x21)},
	{NT_STATUS(0xc0000055), W_ERROR(0x21)},
	{NT_STATUS(0xc0000056), W_ERROR(0x5)},
	{NT_STATUS(0xc0000057), W_ERROR(0x32)},
	{NT_STATUS(0xc0000058), W_ERROR(0x519)},
	{NT_STATUS(0xc0000059), W_ERROR(0x51a)},
	{NT_STATUS(0xc000005a), W_ERROR(0x51b)},
	{NT_STATUS(0xc000005b), W_ERROR(0x51c)},
	{NT_STATUS(0xc000005c), W_ERROR(0x51d)},
	{NT_STATUS(0xc000005d), W_ERROR(0x51e)},
	{NT_STATUS(0xc000005e), W_ERROR(0x51f)},
	{NT_STATUS(0xc000005f), W_ERROR(0x520)},
	{NT_STATUS(0xc0000060), W_ERROR(0x521)},
	{NT_STATUS(0xc0000061), W_ERROR(0x522)},
	{NT_STATUS(0xc0000062), W_ERROR(0x523)},
	{NT_STATUS(0xc0000063), W_ERROR(0x524)},
	{NT_STATUS(0xc0000064), W_ERROR(0x525)},
	{NT_STATUS(0xc0000065), W_ERROR(0x526)},
	{NT_STATUS(0xc0000066), W_ERROR(0x527)},
	{NT_STATUS(0xc0000067), W_ERROR(0x528)},
	{NT_STATUS(0xc0000068), W_ERROR(0x529)},
	{NT_STATUS(0xc0000069), W_ERROR(0x52a)},
	{NT_STATUS(0xc000006a), W_ERROR(0x56)},
	{NT_STATUS(0xc000006b), W_ERROR(0x52c)},
	{NT_STATUS(0xc000006c), W_ERROR(0x52d)},
	{NT_STATUS(0xc000006d), W_ERROR(0x52e)},
	{NT_STATUS(0xc000006e), W_ERROR(0x52f)},
	{NT_STATUS(0xc000006f), W_ERROR(0x530)},
	{NT_STATUS(0xc0000070), W_ERROR(0x531)},
	{NT_STATUS(0xc0000071), W_ERROR(0x532)},
	{NT_STATUS(0xc0000072), W_ERROR(0x533)},
	{NT_STATUS(0xc0000073), W_ERROR(0x534)},
	{NT_STATUS(0xc0000074), W_ERROR(0x535)},
	{NT_STATUS(0xc0000075), W_ERROR(0x536)},
	{NT_STATUS(0xc0000076), W_ERROR(0x537)},
	{NT_STATUS(0xc0000077), W_ERROR(0x538)},
	{NT_STATUS(0xc0000078), W_ERROR(0x539)},
	{NT_STATUS(0xc0000079), W_ERROR(0x53a)},
	{NT_STATUS(0xc000007a), W_ERROR(0x7f)},
	{NT_STATUS(0xc000007b), W_ERROR(0xc1)},
	{NT_STATUS(0xc000007c), W_ERROR(0x3f0)},
	{NT_STATUS(0xc000007d), W_ERROR(0x53c)},
	{NT_STATUS(0xc000007e), W_ERROR(0x9e)},
	{NT_STATUS(0xc000007f), W_ERROR(0x70)},
	{NT_STATUS(0xc0000080), W_ERROR(0x53d)},
	{NT_STATUS(0xc0000081), W_ERROR(0x53e)},
	{NT_STATUS(0xc0000082), W_ERROR(0x44)},
	{NT_STATUS(0xc0000083), W_ERROR(0x103)},
	{NT_STATUS(0xc0000084), W_ERROR(0x53f)},
	{NT_STATUS(0xc0000085), W_ERROR(0x103)},
	{NT_STATUS(0xc0000086), W_ERROR(0x9a)},
	{NT_STATUS(0xc0000087), W_ERROR(0xe)},
	{NT_STATUS(0xc0000088), W_ERROR(0x1e7)},
	{NT_STATUS(0xc0000089), W_ERROR(0x714)},
	{NT_STATUS(0xc000008a), W_ERROR(0x715)},
	{NT_STATUS(0xc000008b), W_ERROR(0x716)},
	{NT_STATUS(0xc000008c), W_ERROR(0xc000008c)},
	{NT_STATUS(0xc000008d), W_ERROR(0xc000008d)},
	{NT_STATUS(0xc000008e), W_ERROR(0xc000008e)},
	{NT_STATUS(0xc000008f), W_ERROR(0xc000008f)},
	{NT_STATUS(0xc0000090), W_ERROR(0xc0000090)},
	{NT_STATUS(0xc0000091), W_ERROR(0xc0000091)},
	{NT_STATUS(0xc0000092), W_ERROR(0xc0000092)},
	{NT_STATUS(0xc0000093), W_ERROR(0xc0000093)},
	{NT_STATUS(0xc0000094), W_ERROR(0xc0000094)},
	{NT_STATUS(0xc0000095), W_ERROR(0x216)},
	{NT_STATUS(0xc0000096), W_ERROR(0xc0000096)},
	{NT_STATUS(0xc0000097), W_ERROR(0x8)},
	{NT_STATUS(0xc0000098), W_ERROR(0x3ee)},
	{NT_STATUS(0xc0000099), W_ERROR(0x540)},
	{NT_STATUS(0xc000009a), W_ERROR(0x5aa)},
	{NT_STATUS(0xc000009b), W_ERROR(0x3)},
	{NT_STATUS(0xc000009c), W_ERROR(0x17)},
	{NT_STATUS(0xc000009d), W_ERROR(0x48f)},
	{NT_STATUS(0xc000009e), W_ERROR(0x15)},
	{NT_STATUS(0xc000009f), W_ERROR(0x1e7)},
	{NT_STATUS(0xc00000a0), W_ERROR(0x1e7)},
	{NT_STATUS(0xc00000a1), W_ERROR(0x5ad)},
	{NT_STATUS(0xc00000a2), W_ERROR(0x13)},
	{NT_STATUS(0xc00000a3), W_ERROR(0x15)},
	{NT_STATUS(0xc00000a4), W_ERROR(0x541)},
	{NT_STATUS(0xc00000a5), W_ERROR(0x542)},
	{NT_STATUS(0xc00000a6), W_ERROR(0x543)},
	{NT_STATUS(0xc00000a7), W_ERROR(0x544)},
	{NT_STATUS(0xc00000a8), W_ERROR(0x545)},
	{NT_STATUS(0xc00000a9), W_ERROR(0x57)},
	{NT_STATUS(0xc00000ab), W_ERROR(0xe7)},
	{NT_STATUS(0xc00000ac), W_ERROR(0xe7)},
	{NT_STATUS(0xc00000ad), W_ERROR(0xe6)},
	{NT_STATUS(0xc00000ae), W_ERROR(0xe7)},
	{NT_STATUS(0xc00000af), W_ERROR(0x1)},
	{NT_STATUS(0xc00000b0), W_ERROR(0xe9)},
	{NT_STATUS(0xc00000b1), W_ERROR(0xe8)},
	{NT_STATUS(0xc00000b2), W_ERROR(0x217)},
	{NT_STATUS(0xc00000b3), W_ERROR(0x218)},
	{NT_STATUS(0xc00000b4), W_ERROR(0xe6)},
	{NT_STATUS(0xc00000b5), W_ERROR(0x79)},
	{NT_STATUS(0xc00000b6), W_ERROR(0x26)},
	{NT_STATUS(0xc00000ba), W_ERROR(0x5)},
	{NT_STATUS(0xc00000bb), W_ERROR(0x32)},
	{NT_STATUS(0xc00000bc), W_ERROR(0x33)},
	{NT_STATUS(0xc00000bd), W_ERROR(0x34)},
	{NT_STATUS(0xc00000be), W_ERROR(0x35)},
	{NT_STATUS(0xc00000bf), W_ERROR(0x36)},
	{NT_STATUS(0xc00000c0), W_ERROR(0x37)},
	{NT_STATUS(0xc00000c1), W_ERROR(0x38)},
	{NT_STATUS(0xc00000c2), W_ERROR(0x39)},
	{NT_STATUS(0xc00000c3), W_ERROR(0x3a)},
	{NT_STATUS(0xc00000c4), W_ERROR(0x3b)},
	{NT_STATUS(0xc00000c5), W_ERROR(0x3c)},
	{NT_STATUS(0xc00000c6), W_ERROR(0x3d)},
	{NT_STATUS(0xc00000c7), W_ERROR(0x3e)},
	{NT_STATUS(0xc00000c8), W_ERROR(0x3f)},
	{NT_STATUS(0xc00000c9), W_ERROR(0x40)},
	{NT_STATUS(0xc00000ca), W_ERROR(0x41)},
	{NT_STATUS(0xc00000cb), W_ERROR(0x42)},
	{NT_STATUS(0xc00000cc), W_ERROR(0x43)},
	{NT_STATUS(0xc00000cd), W_ERROR(0x44)},
	{NT_STATUS(0xc00000ce), W_ERROR(0x45)},
	{NT_STATUS(0xc00000cf), W_ERROR(0x46)},
	{NT_STATUS(0xc00000d0), W_ERROR(0x47)},
	{NT_STATUS(0xc00000d1), W_ERROR(0x48)},
	{NT_STATUS(0xc00000d2), W_ERROR(0x58)},
	{NT_STATUS(0xc00000d4), W_ERROR(0x11)},
	{NT_STATUS(0xc00000d5), W_ERROR(0x5)},
	{NT_STATUS(0xc00000d6), W_ERROR(0xf0)},
	{NT_STATUS(0xc00000d7), W_ERROR(0x546)},
	{NT_STATUS(0xc00000d9), W_ERROR(0xe8)},
	{NT_STATUS(0xc00000da), W_ERROR(0x547)},
	{NT_STATUS(0xc00000dc), W_ERROR(0x548)},
	{NT_STATUS(0xc00000dd), W_ERROR(0x549)},
	{NT_STATUS(0xc00000de), W_ERROR(0x54a)},
	{NT_STATUS(0xc00000df), W_ERROR(0x54b)},
	{NT_STATUS(0xc00000e0), W_ERROR(0x54c)},
	{NT_STATUS(0xc00000e1), W_ERROR(0x54d)},
	{NT_STATUS(0xc00000e2), W_ERROR(0x12c)},
	{NT_STATUS(0xc00000e3), W_ERROR(0x12d)},
	{NT_STATUS(0xc00000e4), W_ERROR(0x54e)},
	{NT_STATUS(0xc00000e5), W_ERROR(0x54f)},
	{NT_STATUS(0xc00000e6), W_ERROR(0x550)},
	{NT_STATUS(0xc00000e7), W_ERROR(0x551)},
	{NT_STATUS(0xc00000e8), W_ERROR(0x6f8)},
	{NT_STATUS(0xc00000ed), W_ERROR(0x552)},
	{NT_STATUS(0xc00000ee), W_ERROR(0x553)},
	{NT_STATUS(0xc00000ef), W_ERROR(0x57)},
	{NT_STATUS(0xc00000f0), W_ERROR(0x57)},
	{NT_STATUS(0xc00000f1), W_ERROR(0x57)},
	{NT_STATUS(0xc00000f2), W_ERROR(0x57)},
	{NT_STATUS(0xc00000f3), W_ERROR(0x57)},
	{NT_STATUS(0xc00000f4), W_ERROR(0x57)},
	{NT_STATUS(0xc00000f5), W_ERROR(0x57)},
	{NT_STATUS(0xc00000f6), W_ERROR(0x57)},
	{NT_STATUS(0xc00000f7), W_ERROR(0x57)},
	{NT_STATUS(0xc00000f8), W_ERROR(0x57)},
	{NT_STATUS(0xc00000f9), W_ERROR(0x57)},
	{NT_STATUS(0xc00000fa), W_ERROR(0x57)},
	{NT_STATUS(0xc00000fb), W_ERROR(0x3)},
	{NT_STATUS(0xc00000fd), W_ERROR(0x3e9)},
	{NT_STATUS(0xc00000fe), W_ERROR(0x554)},
	{NT_STATUS(0xc0000100), W_ERROR(0xcb)},
	{NT_STATUS(0xc0000101), W_ERROR(0x91)},
	{NT_STATUS(0xc0000102), W_ERROR(0x570)},
	{NT_STATUS(0xc0000103), W_ERROR(0x10b)},
	{NT_STATUS(0xc0000104), W_ERROR(0x555)},
	{NT_STATUS(0xc0000105), W_ERROR(0x556)},
	{NT_STATUS(0xc0000106), W_ERROR(0xce)},
	{NT_STATUS(0xc0000107), W_ERROR(0x961)},
	{NT_STATUS(0xc0000108), W_ERROR(0x964)},
	{NT_STATUS(0xc000010a), W_ERROR(0x5)},
	{NT_STATUS(0xc000010b), W_ERROR(0x557)},
	{NT_STATUS(0xc000010d), W_ERROR(0x558)},
	{NT_STATUS(0xc000010e), W_ERROR(0x420)},
	{NT_STATUS(0xc0000117), W_ERROR(0x5a4)},
	{NT_STATUS(0xc000011b), W_ERROR(0xc1)},
	{NT_STATUS(0xc000011c), W_ERROR(0x559)},
	{NT_STATUS(0xc000011d), W_ERROR(0x55a)},
	{NT_STATUS(0xc000011e), W_ERROR(0x3ee)},
	{NT_STATUS(0xc000011f), W_ERROR(0x4)},
	{NT_STATUS(0xc0000120), W_ERROR(0x3e3)},
	{NT_STATUS(0xc0000121), W_ERROR(0x5)},
	{NT_STATUS(0xc0000122), W_ERROR(0x4ba)},
	{NT_STATUS(0xc0000123), W_ERROR(0x5)},
	{NT_STATUS(0xc0000124), W_ERROR(0x55b)},
	{NT_STATUS(0xc0000125), W_ERROR(0x55c)},
	{NT_STATUS(0xc0000126), W_ERROR(0x55d)},
	{NT_STATUS(0xc0000127), W_ERROR(0x55e)},
	{NT_STATUS(0xc0000128), W_ERROR(0x6)},
	{NT_STATUS(0xc000012b), W_ERROR(0x55f)},
	{NT_STATUS(0xc000012d), W_ERROR(0x5af)},
	{NT_STATUS(0xc000012e), W_ERROR(0xc1)},
	{NT_STATUS(0xc000012f), W_ERROR(0xc1)},
	{NT_STATUS(0xc0000130), W_ERROR(0xc1)},
	{NT_STATUS(0xc0000131), W_ERROR(0xc1)},
	{NT_STATUS(0xc0000133), W_ERROR(0x576)},
	{NT_STATUS(0xc0000135), W_ERROR(0x7e)},
	{NT_STATUS(0xc0000138), W_ERROR(0xb6)},
	{NT_STATUS(0xc0000139), W_ERROR(0x7f)},
	{NT_STATUS(0xc000013b), W_ERROR(0x40)},
	{NT_STATUS(0xc000013c), W_ERROR(0x40)},
	{NT_STATUS(0xc000013d), W_ERROR(0x33)},
	{NT_STATUS(0xc000013e), W_ERROR(0x3b)},
	{NT_STATUS(0xc000013f), W_ERROR(0x3b)},
	{NT_STATUS(0xc0000140), W_ERROR(0x3b)},
	{NT_STATUS(0xc0000141), W_ERROR(0x3b)},
	{NT_STATUS(0xc0000142), W_ERROR(0x45a)},
	{NT_STATUS(0xc0000148), W_ERROR(0x7c)},
	{NT_STATUS(0xc0000149), W_ERROR(0x56)},
	{NT_STATUS(0xc000014b), W_ERROR(0x6d)},
	{NT_STATUS(0xc000014c), W_ERROR(0x3f1)},
	{NT_STATUS(0xc000014d), W_ERROR(0x3f8)},
	{NT_STATUS(0xc000014f), W_ERROR(0x3ed)},
	{NT_STATUS(0xc0000150), W_ERROR(0x45e)},
	{NT_STATUS(0xc0000151), W_ERROR(0x560)},
	{NT_STATUS(0xc0000152), W_ERROR(0x561)},
	{NT_STATUS(0xc0000153), W_ERROR(0x562)},
	{NT_STATUS(0xc0000154), W_ERROR(0x563)},
	{NT_STATUS(0xc0000155), W_ERROR(0x564)},
	{NT_STATUS(0xc0000156), W_ERROR(0x565)},
	{NT_STATUS(0xc0000157), W_ERROR(0x566)},
	{NT_STATUS(0xc0000158), W_ERROR(0x567)},
	{NT_STATUS(0xc0000159), W_ERROR(0x3ef)},
	{NT_STATUS(0xc000015a), W_ERROR(0x568)},
	{NT_STATUS(0xc000015b), W_ERROR(0x569)},
	{NT_STATUS(0xc000015c), W_ERROR(0x3f9)},
	{NT_STATUS(0xc000015d), W_ERROR(0x56a)},
	{NT_STATUS(0xc000015f), W_ERROR(0x45d)},
	{NT_STATUS(0xc0000162), W_ERROR(0x459)},
	{NT_STATUS(0xc0000165), W_ERROR(0x462)},
	{NT_STATUS(0xc0000166), W_ERROR(0x463)},
	{NT_STATUS(0xc0000167), W_ERROR(0x464)},
	{NT_STATUS(0xc0000168), W_ERROR(0x465)},
	{NT_STATUS(0xc0000169), W_ERROR(0x466)},
	{NT_STATUS(0xc000016a), W_ERROR(0x467)},
	{NT_STATUS(0xc000016b), W_ERROR(0x468)},
	{NT_STATUS(0xc000016c), W_ERROR(0x45f)},
	{NT_STATUS(0xc000016d), W_ERROR(0x45d)},
	{NT_STATUS(0xc0000172), W_ERROR(0x451)},
	{NT_STATUS(0xc0000173), W_ERROR(0x452)},
	{NT_STATUS(0xc0000174), W_ERROR(0x453)},
	{NT_STATUS(0xc0000175), W_ERROR(0x454)},
	{NT_STATUS(0xc0000176), W_ERROR(0x455)},
	{NT_STATUS(0xc0000177), W_ERROR(0x469)},
	{NT_STATUS(0xc0000178), W_ERROR(0x458)},
	{NT_STATUS(0xc000017a), W_ERROR(0x56b)},
	{NT_STATUS(0xc000017b), W_ERROR(0x56c)},
	{NT_STATUS(0xc000017c), W_ERROR(0x3fa)},
	{NT_STATUS(0xc000017d), W_ERROR(0x3fb)},
	{NT_STATUS(0xc000017e), W_ERROR(0x56d)},
	{NT_STATUS(0xc000017f), W_ERROR(0x56e)},
	{NT_STATUS(0xc0000180), W_ERROR(0x3fc)},
	{NT_STATUS(0xc0000181), W_ERROR(0x3fd)},
	{NT_STATUS(0xc0000182), W_ERROR(0x57)},
	{NT_STATUS(0xc0000183), W_ERROR(0x45d)},
	{NT_STATUS(0xc0000184), W_ERROR(0x16)},
	{NT_STATUS(0xc0000185), W_ERROR(0x45d)},
	{NT_STATUS(0xc0000186), W_ERROR(0x45d)},
	{NT_STATUS(0xc0000188), W_ERROR(0x5de)},
	{NT_STATUS(0xc0000189), W_ERROR(0x13)},
	{NT_STATUS(0xc000018a), W_ERROR(0x6fa)},
	{NT_STATUS(0xc000018b), W_ERROR(0x6fb)},
	{NT_STATUS(0xc000018c), W_ERROR(0x6fc)},
	{NT_STATUS(0xc000018d), W_ERROR(0x6fd)},
	{NT_STATUS(0xc000018e), W_ERROR(0x5dc)},
	{NT_STATUS(0xc000018f), W_ERROR(0x5dd)},
	{NT_STATUS(0xc0000190), W_ERROR(0x6fe)},
	{NT_STATUS(0xc0000192), W_ERROR(0x700)},
	{NT_STATUS(0xc0000193), W_ERROR(0x701)},
	{NT_STATUS(0xc0000194), W_ERROR(0x46b)},
	{NT_STATUS(0xc0000195), W_ERROR(0x4c3)},
	{NT_STATUS(0xc0000196), W_ERROR(0x4c4)},
	{NT_STATUS(0xc0000197), W_ERROR(0x5df)},
	{NT_STATUS(0xc0000198), W_ERROR(0x70f)},
	{NT_STATUS(0xc0000199), W_ERROR(0x710)},
	{NT_STATUS(0xc000019a), W_ERROR(0x711)},
	{NT_STATUS(0xc000019b), W_ERROR(0x712)},
	{NT_STATUS(0xc0000202), W_ERROR(0x572)},
	{NT_STATUS(0xc0000203), W_ERROR(0x3b)},
	{NT_STATUS(0xc0000204), W_ERROR(0x717)},
	{NT_STATUS(0xc0000205), W_ERROR(0x46a)},
	{NT_STATUS(0xc0000206), W_ERROR(0x6f8)},
	{NT_STATUS(0xc0000207), W_ERROR(0x4be)},
	{NT_STATUS(0xc0000208), W_ERROR(0x4be)},
	{NT_STATUS(0xc0000209), W_ERROR(0x44)},
	{NT_STATUS(0xc000020a), W_ERROR(0x34)},
	{NT_STATUS(0xc000020b), W_ERROR(0x40)},
	{NT_STATUS(0xc000020c), W_ERROR(0x40)},
	{NT_STATUS(0xc000020d), W_ERROR(0x40)},
	{NT_STATUS(0xc000020e), W_ERROR(0x44)},
	{NT_STATUS(0xc000020f), W_ERROR(0x3b)},
	{NT_STATUS(0xc0000210), W_ERROR(0x3b)},
	{NT_STATUS(0xc0000211), W_ERROR(0x3b)},
	{NT_STATUS(0xc0000212), W_ERROR(0x3b)},
	{NT_STATUS(0xc0000213), W_ERROR(0x3b)},
	{NT_STATUS(0xc0000214), W_ERROR(0x3b)},
	{NT_STATUS(0xc0000215), W_ERROR(0x3b)},
	{NT_STATUS(0xc0000216), W_ERROR(0x32)},
	{NT_STATUS(0xc0000217), W_ERROR(0x32)},
	{NT_STATUS(0xc000021c), W_ERROR(0x17e6)},
	{NT_STATUS(0xc0000220), W_ERROR(0x46c)},
	{NT_STATUS(0xc0000221), W_ERROR(0xc1)},
	{NT_STATUS(0xc0000224), W_ERROR(0x773)},
	{NT_STATUS(0xc0000225), W_ERROR(0x490)},
	{NT_STATUS(0xc000022a), W_ERROR(0xc000022a)},
	{NT_STATUS(0xc000022b), W_ERROR(0xc000022b)},
	{NT_STATUS(0xc000022d), W_ERROR(0x4d5)},
	{NT_STATUS(0xc0000230), W_ERROR(0x492)},
	{NT_STATUS(0xc0000233), W_ERROR(0x774)},
	{NT_STATUS(0xc0000234), W_ERROR(0x775)},
	{NT_STATUS(0xc0000235), W_ERROR(0x6)},
	{NT_STATUS(0xc0000236), W_ERROR(0x4c9)},
	{NT_STATUS(0xc0000237), W_ERROR(0x4ca)},
	{NT_STATUS(0xc0000238), W_ERROR(0x4cb)},
	{NT_STATUS(0xc0000239), W_ERROR(0x4cc)},
	{NT_STATUS(0xc000023a), W_ERROR(0x4cd)},
	{NT_STATUS(0xc000023b), W_ERROR(0x4ce)},
	{NT_STATUS(0xc000023c), W_ERROR(0x4cf)},
	{NT_STATUS(0xc000023d), W_ERROR(0x4d0)},
	{NT_STATUS(0xc000023e), W_ERROR(0x4d1)},
	{NT_STATUS(0xc000023f), W_ERROR(0x4d2)},
	{NT_STATUS(0xc0000240), W_ERROR(0x4d3)},
	{NT_STATUS(0xc0000241), W_ERROR(0x4d4)},
	{NT_STATUS(0xc0000243), W_ERROR(0x4c8)},
	{NT_STATUS(0xc0000246), W_ERROR(0x4d6)},
	{NT_STATUS(0xc0000247), W_ERROR(0x4d7)},
	{NT_STATUS(0xc0000248), W_ERROR(0x4d8)},
	{NT_STATUS(0xc0000249), W_ERROR(0xc1)},
	{NT_STATUS(0xc0000253), W_ERROR(0x54f)},
	{NT_STATUS(0xc0000257), W_ERROR(0x4d0)},
	{NT_STATUS(0xc0000259), W_ERROR(0x573)},
	{NT_STATUS(0xc000025e), W_ERROR(0x422)},
	{NT_STATUS(0xc0000262), W_ERROR(0xb6)},
	{NT_STATUS(0xc0000263), W_ERROR(0x7f)},
	{NT_STATUS(0xc0000264), W_ERROR(0x120)},
	{NT_STATUS(0xc0000265), W_ERROR(0x476)},
	{NT_STATUS(0xc0000267), W_ERROR(0x10fe)},
	{NT_STATUS(0xc000026c), W_ERROR(0x7d1)},
	{NT_STATUS(0xc000026d), W_ERROR(0x4b1)},
	{NT_STATUS(0xc000026e), W_ERROR(0x15)},
	{NT_STATUS(0xc0000272), W_ERROR(0x491)},
	{NT_STATUS(0xc0000275), W_ERROR(0x1126)},
	{NT_STATUS(0xc0000276), W_ERROR(0x1129)},
	{NT_STATUS(0xc0000277), W_ERROR(0x112a)},
	{NT_STATUS(0xc0000278), W_ERROR(0x1128)},
	{NT_STATUS(0xc0000279), W_ERROR(0x780)},
	{NT_STATUS(0xc0000280), W_ERROR(0x781)},
	{NT_STATUS(0xc0000281), W_ERROR(0xa1)},
	{NT_STATUS(0xc0000283), W_ERROR(0x488)},
	{NT_STATUS(0xc0000284), W_ERROR(0x489)},
	{NT_STATUS(0xc0000285), W_ERROR(0x48a)},
	{NT_STATUS(0xc0000286), W_ERROR(0x48b)},
	{NT_STATUS(0xc0000287), W_ERROR(0x48c)},
	{NT_STATUS(0xc000028a), W_ERROR(0x5)},
	{NT_STATUS(0xc000028b), W_ERROR(0x5)},
	{NT_STATUS(0xc000028d), W_ERROR(0x5)},
	{NT_STATUS(0xc000028e), W_ERROR(0x5)},
	{NT_STATUS(0xc000028f), W_ERROR(0x5)},
	{NT_STATUS(0xc0000290), W_ERROR(0x5)},
	{NT_STATUS(0xc0000291), W_ERROR(0x1777)},
	{NT_STATUS(0xc0000292), W_ERROR(0x1778)},
	{NT_STATUS(0xc0000293), W_ERROR(0x1772)},
	{NT_STATUS(0xc0000295), W_ERROR(0x1068)},
	{NT_STATUS(0xc0000296), W_ERROR(0x1069)},
	{NT_STATUS(0xc0000297), W_ERROR(0x106a)},
	{NT_STATUS(0xc0000298), W_ERROR(0x106b)},
	{NT_STATUS(0xc0000299), W_ERROR(0x201a)},
	{NT_STATUS(0xc000029a), W_ERROR(0x201b)},
	{NT_STATUS(0xc000029b), W_ERROR(0x201c)},
	{NT_STATUS(0xc000029c), W_ERROR(0x1)},
	{NT_STATUS(0xc000029d), W_ERROR(0x10ff)},
	{NT_STATUS(0xc000029e), W_ERROR(0x1100)},
	{NT_STATUS(0xc000029f), W_ERROR(0x494)},
	{NT_STATUS(0xc00002a1), W_ERROR(0x200a)},
	{NT_STATUS(0xc00002a2), W_ERROR(0x200b)},
	{NT_STATUS(0xc00002a3), W_ERROR(0x200c)},
	{NT_STATUS(0xc00002a4), W_ERROR(0x200d)},
	{NT_STATUS(0xc00002a5), W_ERROR(0x200e)},
	{NT_STATUS(0xc00002a6), W_ERROR(0x200f)},
	{NT_STATUS(0xc00002a7), W_ERROR(0x2010)},
	{NT_STATUS(0xc00002a8), W_ERROR(0x2011)},
	{NT_STATUS(0xc00002a9), W_ERROR(0x2012)},
	{NT_STATUS(0xc00002aa), W_ERROR(0x2013)},
	{NT_STATUS(0xc00002ab), W_ERROR(0x2014)},
	{NT_STATUS(0xc00002ac), W_ERROR(0x2015)},
	{NT_STATUS(0xc00002ad), W_ERROR(0x2016)},
	{NT_STATUS(0xc00002ae), W_ERROR(0x2017)},
	{NT_STATUS(0xc00002af), W_ERROR(0x2018)},
	{NT_STATUS(0xc00002b0), W_ERROR(0x2019)},
	{NT_STATUS(0xc00002b1), W_ERROR(0x211e)},
	{NT_STATUS(0xc00002b2), W_ERROR(0x1127)},
	{NT_STATUS(0xc00002b6), W_ERROR(0x651)},
	{NT_STATUS(0xc00002b7), W_ERROR(0x49a)},
	{NT_STATUS(0xc00002b8), W_ERROR(0x49b)},
	{NT_STATUS(0xc00002c1), W_ERROR(0x2024)},
	{NT_STATUS(0xc00002c3), W_ERROR(0x575)},
	{NT_STATUS(0xc00002c5), W_ERROR(0x3e6)},
	{NT_STATUS(0xc00002c6), W_ERROR(0x1075)},
	{NT_STATUS(0xc00002c7), W_ERROR(0x1076)},
	{NT_STATUS(0xc00002ca), W_ERROR(0x10e8)},
	{NT_STATUS(0xc00002cb), W_ERROR(0x2138)},
	{NT_STATUS(0xc00002cc), W_ERROR(0x4e3)},
	{NT_STATUS(0xc00002cd), W_ERROR(0x2139)},
	{NT_STATUS(0xc00002cf), W_ERROR(0x49d)},
	{NT_STATUS(0xc00002d0), W_ERROR(0x213a)},
	{NT_STATUS(0xc00002d4), W_ERROR(0x2141)},
	{NT_STATUS(0xc00002d5), W_ERROR(0x2142)},
	{NT_STATUS(0xc00002d6), W_ERROR(0x2143)},
	{NT_STATUS(0xc00002d7), W_ERROR(0x2144)},
	{NT_STATUS(0xc00002d8), W_ERROR(0x2145)},
	{NT_STATUS(0xc00002d9), W_ERROR(0x2146)},
	{NT_STATUS(0xc00002da), W_ERROR(0x2147)},
	{NT_STATUS(0xc00002db), W_ERROR(0x2148)},
	{NT_STATUS(0xc00002dc), W_ERROR(0x2149)},
	{NT_STATUS(0xc00002dd), W_ERROR(0x32)},
	{NT_STATUS(0xc00002df), W_ERROR(0x2151)},
	{NT_STATUS(0xc00002e0), W_ERROR(0x2152)},
	{NT_STATUS(0xc00002e1), W_ERROR(0x2153)},
	{NT_STATUS(0xc00002e2), W_ERROR(0x2154)},
	{NT_STATUS(0xc00002e3), W_ERROR(0x215d)},
	{NT_STATUS(0xc00002e4), W_ERROR(0x2163)},
	{NT_STATUS(0xc00002e5), W_ERROR(0x2164)},
	{NT_STATUS(0xc00002e6), W_ERROR(0x2165)},
	{NT_STATUS(0xc00002e7), W_ERROR(0x216d)},
	{NT_STATUS(0xc00002fe), W_ERROR(0x45b)},
	{NT_STATUS(0xc00002ff), W_ERROR(0x4e7)},
	{NT_STATUS(0xc0000300), W_ERROR(0x4e6)},
	{NT_STATUS(0x80000001), W_ERROR(0x80000001)},
	{NT_STATUS(0x80000002), W_ERROR(0x3e6)},
	{NT_STATUS(0x80000003), W_ERROR(0x80000003)},
	{NT_STATUS(0x80000004), W_ERROR(0x80000004)},
	{NT_STATUS(0x80000005), W_ERROR(0xea)},
	{NT_STATUS(0x80000006), W_ERROR(0x12)},
	{NT_STATUS(0x8000000b), W_ERROR(0x56f)},
	{NT_STATUS(0x8000000d), W_ERROR(0x12b)},
	{NT_STATUS(0x8000000e), W_ERROR(0x1c)},
	{NT_STATUS(0x8000000f), W_ERROR(0x15)},
	{NT_STATUS(0x80000010), W_ERROR(0x15)},
	{NT_STATUS(0x80000011), W_ERROR(0xaa)},
	{NT_STATUS(0x80000012), W_ERROR(0x103)},
	{NT_STATUS(0x80000013), W_ERROR(0xfe)},
	{NT_STATUS(0x80000014), W_ERROR(0xff)},
	{NT_STATUS(0x80000015), W_ERROR(0xff)},
	{NT_STATUS(0x80000016), W_ERROR(0x456)},
	{NT_STATUS(0x8000001a), W_ERROR(0x103)},
	{NT_STATUS(0x8000001b), W_ERROR(0x44d)},
	{NT_STATUS(0x8000001c), W_ERROR(0x456)},
	{NT_STATUS(0x8000001d), W_ERROR(0x457)},
	{NT_STATUS(0x8000001e), W_ERROR(0x44c)},
	{NT_STATUS(0x8000001f), W_ERROR(0x44e)},
	{NT_STATUS(0x80000021), W_ERROR(0x44f)},
	{NT_STATUS(0x80000022), W_ERROR(0x450)},
	{NT_STATUS(0x80000025), W_ERROR(0x962)},
	{NT_STATUS(0x80000288), W_ERROR(0x48d)},
	{NT_STATUS(0x80000289), W_ERROR(0x48e)},
	{NT_STATUS_OK, WERR_OK}};


/*****************************************************************************
convert a dos eclas/ecode to a NT status32 code
 *****************************************************************************/
NTSTATUS dos_to_ntstatus(int eclass, int ecode)
{
	int i;
	if (eclass == 0 && ecode == 0) return NT_STATUS_OK;
	for (i=0; NT_STATUS_V(dos_to_ntstatus_map[i].ntstatus); i++) {
		if (eclass == dos_to_ntstatus_map[i].dos_class &&
		    ecode == dos_to_ntstatus_map[i].dos_code) {
			return dos_to_ntstatus_map[i].ntstatus;
		}
	}
	return NT_STATUS_UNSUCCESSFUL;
}


/*****************************************************************************
convert a NT status code to a dos class/code
 *****************************************************************************/
void ntstatus_to_dos(NTSTATUS ntstatus, uint8 *eclass, uint32 *ecode)
{
	int i;
	if (NT_STATUS_IS_OK(ntstatus)) {
		*eclass = 0;
		*ecode = 0;
		return;
	}
	for (i=0; NT_STATUS_V(ntstatus_to_dos_map[i].ntstatus); i++) {
		if (NT_STATUS_V(ntstatus) == 
		    NT_STATUS_V(ntstatus_to_dos_map[i].ntstatus)) {
			*eclass = ntstatus_to_dos_map[i].dos_class;
			*ecode = ntstatus_to_dos_map[i].dos_code;
			return;
		}
	}
	*eclass = ERRSRV;
	*ecode = ERRerror;
}


/*****************************************************************************
convert a WERROR to a NT status32 code
 *****************************************************************************/
NTSTATUS werror_to_ntstatus(WERROR error)
{
	int i;
	if (W_ERROR_IS_OK(error)) return NT_STATUS_OK;
	for (i=0; NT_STATUS_V(ntstatus_to_werror_map[i].ntstatus); i++) {
		if (W_ERROR_V(error) == 
		    W_ERROR_V(ntstatus_to_werror_map[i].werror)) {
			return ntstatus_to_werror_map[i].ntstatus;
		}
	}

	/* just guess ... */
	return NT_STATUS(W_ERROR_V(error) | 0xc0000000);
}

/*****************************************************************************
convert a NTSTATUS to a WERROR
 *****************************************************************************/
WERROR ntstatus_to_werror(NTSTATUS error)
{
	int i;
	if (NT_STATUS_IS_OK(error)) return WERR_OK;
	for (i=0; NT_STATUS_V(ntstatus_to_werror_map[i].ntstatus); i++) {
		if (NT_STATUS_V(error) == 
		    NT_STATUS_V(ntstatus_to_werror_map[i].ntstatus)) {
			return ntstatus_to_werror_map[i].werror;
		}
	}

	/* a lame guess */
	return W_ERROR(NT_STATUS_V(error) & 0xffff);
}
